# -*- coding: utf-8 -*-
#
# Author: Natalia Bidart <natalia.bidart@canonical.com>
# Author: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2010-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Tests for the SSO account code."""

from __future__ import unicode_literals

import os

# pylint: disable=W0621,F0401,E0611
try:
    import urllib.request as url_lib
except ImportError:
    import urllib2 as url_lib
# pylint: enable=W0621,F0401,E0611

from twisted.trial.unittest import TestCase
from twisted.internet import defer

from ubuntu_kylin_sso import account
from ubuntu_kylin_sso.account import (
    Account,
    AuthenticationError,
    EmailTokenError,
    InvalidEmailError,
    InvalidPasswordError,
    NewPasswordError,
    SERVICE_URL,
    RegistrationError,
    ResetPasswordTokenError,
    SSO_STATUS_OK,
    SSO_STATUS_ERROR,
)
from ubuntu_kylin_sso.tests import (
    APP_NAME,
    CAPTCHA_ID,
    CAPTCHA_PATH,
    CAPTCHA_SOLUTION,
    EMAIL,
    EMAIL_TOKEN,
    NAME,
    PASSWORD,
    RESET_PASSWORD_TOKEN,
    TOKEN,
    TOKEN_NAME,
)
from ubuntu_kylin_sso.utils import compat


CANT_RESET_PASSWORD_CONTENT = "CanNotResetPassowrdError: " \
                              "Can't reset password for this account"
RESET_TOKEN_INVALID_CONTENT = "AuthToken matching query does not exist."
EMAIL_ALREADY_REGISTERED = 'a@example.com'
STATUS_UNKNOWN = {'status': 'yadda-yadda'}
STATUS_ERROR = {'status': SSO_STATUS_ERROR,
                'errors': {'something': ['Bla', 'Ble']}}
STATUS_OK = {'status': SSO_STATUS_OK}
STATUS_EMAIL_UNKNOWN = {'status': 'yadda-yadda'}
STATUS_EMAIL_ERROR = {'errors': {'email_token': ['Error1', 'Error2']}}
STATUS_EMAIL_OK = {'email': EMAIL}

FAKE_NEW_CAPTCHA = {
    'image_url': "file://" + compat.text_type(CAPTCHA_PATH),
    'captcha_id': CAPTCHA_ID,
}


class FakeWebClientResponse(object):
    """A fake webclient.Response."""

    content = open(CAPTCHA_PATH, "rb").read()


class FakeWebClient(object):
    """A fake webclient."""

    def __init__(self):
        self.started = True

    def request(self, url):
        """Do a fake request, return a fake Response."""
        return FakeWebClientResponse()

    def shutdown(self):
        """Turn off this webclient."""
        self.started = False


class FakeRestfulClient(object):
    """A fake restfulclient."""

    preferred_email = EMAIL

    def __init__(self):
        self.started = True

    def shutdown(self):
        """Stop this restfulclient."""
        self.started = False

    def fake_captchas_new(self):
        """Return a local fake captcha."""
        return FAKE_NEW_CAPTCHA

    def fake_registration_register(self, email, password, displayname,
                                    captcha_id, captcha_solution):
        """Fake registration. Return a fix result."""
        if email == EMAIL_ALREADY_REGISTERED:
            return {'status': SSO_STATUS_ERROR,
                    'errors': {'email': 'Email already registered'}}
        elif captcha_id is None and captcha_solution is None:
            return STATUS_UNKNOWN
        elif captcha_id != CAPTCHA_ID or captcha_solution != CAPTCHA_SOLUTION:
            return STATUS_ERROR
        else:
            return STATUS_OK

    def fake_registration_request_password_reset_token(self, email):
        """Fake password reset token. Return a fix result."""
        if email is None:
            return STATUS_UNKNOWN
        elif email != EMAIL:
            raise account.WebClientError("Misc error",
                                         CANT_RESET_PASSWORD_CONTENT)
        else:
            return STATUS_OK

    def fake_registration_set_new_password(self, email, token, new_password):
        """Fake the setting of new password. Return a fix result."""
        if email is None and token is None and new_password is None:
            return STATUS_UNKNOWN
        elif email != EMAIL or token != RESET_PASSWORD_TOKEN:
            raise account.WebClientError("Misc error",
                                         RESET_TOKEN_INVALID_CONTENT)
        else:
            return STATUS_OK

    def fake_authentications_authenticate(self, token_name):
        """Fake authenticate. Return a fix result."""
        if not token_name.startswith(TOKEN_NAME):
            raise account.WebClientError()
        else:
            return TOKEN

    def fake_accounts_validate_email(self, email_token):
        """Fake the email validation. Return a fix result."""
        if email_token is None:
            return STATUS_EMAIL_UNKNOWN
        elif email_token == EMAIL_ALREADY_REGISTERED:
            return {
                'status': SSO_STATUS_ERROR,
                'errors': {'email': 'Email already registered'}
            }
        elif email_token != EMAIL_TOKEN:
            return STATUS_EMAIL_ERROR
        else:
            return STATUS_EMAIL_OK

    def fake_accounts_me(self):
        """Fake the 'me' information."""
        return {'username': 'Wh46bKY',
                'preferred_email': self.preferred_email,
                'displayname': '',
                'unverified_emails': ['aaaaaa@example.com'],
                'verified_emails': [],
                'openid_identifier': 'Wh46bKY'}

    def check_all_kwargs_unicode(self, **kwargs):
        """Check that the values of all keyword arguments are unicode."""
        for (key, value) in kwargs.items():
            if isinstance(value, compat.binary_type):
                raise AssertionError("Error: kwarg '%s' is non-unicode." % key)

    def restcall(self, method_name, **kwargs):
        """Fake an async restcall."""
        self.check_all_kwargs_unicode(**kwargs)
        method = getattr(self, "fake_" + method_name.replace(".", "_"))
        try:
            return defer.succeed(method(**kwargs))
        # pylint: disable=W0703
        except Exception as e:
            return defer.fail(e)


class AccountTestCase(TestCase):
    """Test suite for the SSO login processor."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set up."""
        yield super(AccountTestCase, self).setUp()

        def fake_urlopen(url):
            """Fake an urlopen which will read from the disk."""
            f = open(url)
            self.addCleanup(f.close)
            return f

        self.patch(url_lib, 'urlopen', fake_urlopen)  # fd to the path
        self.processor = Account()
        self.register_kwargs = dict(email=EMAIL, password=PASSWORD,
                                    displayname=NAME,
                                    captcha_id=CAPTCHA_ID,
                                    captcha_solution=CAPTCHA_SOLUTION)
        self.login_kwargs = dict(email=EMAIL, password=PASSWORD,
                                 token_name=TOKEN_NAME)
        self.frc = FakeRestfulClient()
        self.patch(account.restful, "RestfulClient",
                   lambda *args, **kwargs: self.frc)
        self.addCleanup(self.verify_frc_shutdown)

    def verify_frc_shutdown(self):
        """Verify that the FakeRestfulClient was stopped."""
        assert self.frc.started is False, "Restfulclient must be shut down."

    @defer.inlineCallbacks
    def test_generate_captcha(self):
        """Captcha can be generated."""
        filename = self.mktemp()
        self.addCleanup(lambda: os.remove(filename)
                                if os.path.exists(filename) else None)
        wc = FakeWebClient()
        self.patch(account.webclient, "webclient_factory", lambda: wc)
        captcha_id = yield self.processor.generate_captcha(filename)
        self.assertEqual(CAPTCHA_ID, captcha_id, 'captcha id must be correct.')
        self.assertTrue(os.path.isfile(filename), '%s must exist.' % filename)

        with open(CAPTCHA_PATH) as f:
            expected = f.read()
        with open(filename) as f:
            actual = f.read()
        self.assertEqual(expected, actual, 'captcha image must be correct.')
        self.assertFalse(wc.started, "Webclient must be shut down.")

    @defer.inlineCallbacks
    def test_register_user_checks_valid_email(self):
        """Email is validated."""
        self.register_kwargs['email'] = 'notavalidemail'
        d = self.processor.register_user(**self.register_kwargs)
        yield self.assertFailure(d, InvalidEmailError)

    @defer.inlineCallbacks
    def test_register_user_checks_valid_password(self):
        """Password is validated."""
        self.register_kwargs['password'] = ''
        d = self.processor.register_user(**self.register_kwargs)
        yield self.assertFailure(d, InvalidPasswordError)

        # 7 chars, one less than expected
        self.register_kwargs['password'] = 'tesT3it'
        d = self.processor.register_user(**self.register_kwargs)
        yield self.assertFailure(d, InvalidPasswordError)

        self.register_kwargs['password'] = 'test3it!'  # no upper case
        d = self.processor.register_user(**self.register_kwargs)
        yield self.assertFailure(d, InvalidPasswordError)

        self.register_kwargs['password'] = 'testIt!!'  # no number
        d = self.processor.register_user(**self.register_kwargs)
        yield self.assertFailure(d, InvalidPasswordError)

    # register

    @defer.inlineCallbacks
    def test_register_user_if_status_ok(self):
        """A user is succesfuy registered into the SSO server."""
        result = yield self.processor.register_user(**self.register_kwargs)
        self.assertEqual(EMAIL, result, 'registration was successful.')

    @defer.inlineCallbacks
    def test_register_user_if_status_error(self):
        """Proper error is raised if register fails."""
        self.register_kwargs['captcha_id'] = CAPTCHA_ID * 2  # incorrect
        d = self.processor.register_user(**self.register_kwargs)
        failure = yield self.assertFailure(d, RegistrationError)
        for k, val in failure.args[0].items():
            self.assertIn(k, STATUS_ERROR['errors'])
            self.assertEqual(val, "\n".join(STATUS_ERROR['errors'][k]))

    @defer.inlineCallbacks
    def test_register_user_if_status_error_with_string_message(self):
        """Proper error is raised if register fails."""
        self.register_kwargs['email'] = EMAIL_ALREADY_REGISTERED
        d = self.processor.register_user(**self.register_kwargs)
        failure = yield self.assertFailure(d, RegistrationError)
        for k, val in failure.args[0].items():
            self.assertIn(k, {'email': 'Email already registered'})
            self.assertEqual(val, 'Email already registered')

    @defer.inlineCallbacks
    def test_register_user_if_status_unknown(self):
        """Proper error is raised if register returns an unknown status."""
        self.register_kwargs['captcha_id'] = None
        self.register_kwargs['captcha_solution'] = None
        d = self.processor.register_user(**self.register_kwargs)
        failure = yield self.assertFailure(d, RegistrationError)
        self.assertIn('Received unknown status: %s' % STATUS_UNKNOWN, failure)

    # login

    @defer.inlineCallbacks
    def test_login_if_http_error(self):
        """Proper error is raised if authentication fails."""
        # use an invalid token name
        self.login_kwargs['token_name'] = APP_NAME * 2
        d = self.processor.login(**self.login_kwargs)
        yield self.assertFailure(d, AuthenticationError)

    @defer.inlineCallbacks
    def test_login_if_no_error(self):
        """A user can be succesfully logged in into the SSO service."""
        result = yield self.processor.login(**self.login_kwargs)
        self.assertEqual(TOKEN, result, 'authentication was successful.')

    # is_validated

    @defer.inlineCallbacks
    def test_is_validated(self):
        """If preferred email is not None, user is validated."""
        result = yield self.processor.is_validated(token=TOKEN)
        self.assertTrue(result, 'user must be validated.')

    @defer.inlineCallbacks
    def test_is_not_validated(self):
        """If preferred email is None, user is not validated."""
        self.frc.preferred_email = None
        result = yield self.processor.is_validated(token=TOKEN)
        self.assertFalse(result, 'user must not be validated.')

    @defer.inlineCallbacks
    def test_is_not_validated_empty_result(self):
        """If preferred email is None, user is not validated."""
        self.patch(self.frc, "fake_accounts_me", lambda *args: {})
        result = yield self.processor.is_validated(token=TOKEN)
        self.assertFalse(result, 'user must not be validated.')

    # validate_email

    @defer.inlineCallbacks
    def test_validate_email_if_status_ok(self):
        """A email is succesfuy validated in the SSO server."""
        self.login_kwargs['email_token'] = EMAIL_TOKEN  # valid email token
        result = yield self.processor.validate_email(**self.login_kwargs)
        self.assertEqual(TOKEN, result, 'email validation was successful.')

    @defer.inlineCallbacks
    def test_validate_email_if_status_error(self):
        """Proper error is raised if email validation fails."""
        self.login_kwargs['email_token'] = EMAIL_TOKEN * 2  # invalid token
        d = self.processor.validate_email(**self.login_kwargs)
        failure = yield self.assertFailure(d, EmailTokenError)
        for k, val in failure.args[0].items():
            self.assertIn(k, STATUS_EMAIL_ERROR['errors'])
            self.assertEqual(val, "\n".join(STATUS_EMAIL_ERROR['errors'][k]))

    @defer.inlineCallbacks
    def test_validate_email_if_status_error_with_string_message(self):
        """Proper error is raised if register fails."""
        self.login_kwargs['email_token'] = EMAIL_ALREADY_REGISTERED
        d = self.processor.validate_email(**self.login_kwargs)
        failure = yield self.assertFailure(d, EmailTokenError)
        for k, val in failure.args[0].items():
            self.assertIn(k, {'email': 'Email already registered'})
            self.assertEqual(val, 'Email already registered')

    @defer.inlineCallbacks
    def test_validate_email_if_status_unknown(self):
        """Proper error is raised if email validation returns unknown."""
        self.login_kwargs['email_token'] = None
        d = self.processor.validate_email(**self.login_kwargs)
        failure = yield self.assertFailure(d, EmailTokenError)
        self.assertIn('Received invalid reply: %s' % STATUS_UNKNOWN, failure)

    # reset_password

    @defer.inlineCallbacks
    def test_request_password_reset_token_if_status_ok(self):
        """A reset password token is succesfuly sent."""
        result = yield self.processor.request_password_reset_token(email=EMAIL)
        self.assertEqual(EMAIL, result,
                         'password reset token must be successful.')

    @defer.inlineCallbacks
    def test_request_password_reset_token_if_http_error(self):
        """Proper error is raised if password token request fails."""
        d = self.processor.request_password_reset_token(email=EMAIL * 2)
        exc = yield self.assertFailure(d, ResetPasswordTokenError)
        self.assertIn(CANT_RESET_PASSWORD_CONTENT, exc)

    @defer.inlineCallbacks
    def test_request_password_reset_token_if_status_unknown(self):
        """Proper error is raised if password token request returns unknown."""
        d = self.processor.request_password_reset_token(email=None)
        exc = yield self.assertFailure(d, ResetPasswordTokenError)
        self.assertIn('Received invalid reply: %s' % STATUS_UNKNOWN, exc)

    @defer.inlineCallbacks
    def test_set_new_password_if_status_ok(self):
        """A new password is succesfuy set."""
        result = yield self.processor.set_new_password(email=EMAIL,
                                                 token=RESET_PASSWORD_TOKEN,
                                                 new_password=PASSWORD)
        self.assertEqual(EMAIL, result,
                         'new password must be set successfully.')

    @defer.inlineCallbacks
    def test_set_new_password_if_http_error(self):
        """Proper error is raised if setting a new password fails."""
        d = self.processor.set_new_password(email=EMAIL * 2,
                                            token=RESET_PASSWORD_TOKEN * 2,
                                            new_password=PASSWORD)
        exc = yield self.assertFailure(d, NewPasswordError)
        self.assertIn(RESET_TOKEN_INVALID_CONTENT, exc)

    @defer.inlineCallbacks
    def test_set_new_password_if_status_unknown(self):
        """Proper error is raised if setting a new password returns unknown."""
        d = self.processor.set_new_password(email=None, token=None,
                                            new_password=None)
        exc = yield self.assertFailure(d, NewPasswordError)
        self.assertIn('Received invalid reply: %s' % STATUS_UNKNOWN, exc)


class EnvironOverridesTestCase(TestCase):
    """Some URLs can be set from the environment for testing/QA purposes."""

    def test_override_service_url(self):
        """The service url can be set from the env var USSOC_SERVICE_URL."""
        fake_url = 'this is not really a URL, but ends with slash: /'
        old_url = os.environ.get('USSOC_SERVICE_URL')
        os.environ['USSOC_SERVICE_URL'] = fake_url
        try:
            proc = Account()
            self.assertEqual(proc.service_url, fake_url)
        finally:
            if old_url:
                os.environ['USSOC_SERVICE_URL'] = old_url
            else:
                del os.environ['USSOC_SERVICE_URL']

    def test_no_override_service_url(self):
        """If the environ is unset, the default service url is used."""
        proc = Account()
        self.assertEqual(proc.service_url, SERVICE_URL)

    def test_service_url_as_parameter(self):
        """If the parameter service url is given, is used."""
        expected = 'http://foo/bar/baz/'
        proc = Account(service_url=expected)
        self.assertEqual(proc.service_url, expected)



# -*- coding: utf-8 -*-
#
# Copyright 2014 National University of Defense Technology(NUDT) & Kylin Ltd
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Implementations of different keyrings."""

from __future__ import unicode_literals

import socket
import sys

try:
    # pylint: disable=E0611,F0401
    from urllib.parse import quote
    # pylint: enable=E0611,F0401
except ImportError:
    from urllib import quote

from twisted.internet.defer import inlineCallbacks, returnValue

from ubuntu_kylin_sso.logger import setup_logging
from ubuntu_kylin_sso.utils import compat

logger = setup_logging("ubuntu_kylin_sso.keyring")

TOKEN_SEPARATOR = ' @ '
SEPARATOR_REPLACEMENT = ' AT '

U1_APP_NAME = "Ubuntu One"
U1_KEY_NAME = "UbuntuOne token for https://ubuntuone.com"
U1_KEY_ATTR = {
    "oauth-consumer-key": "ubuntuone",
    "ubuntuone-realm": "https://ubuntuone.com",
}


def gethostname():
    """Get the hostname, return the name as unicode."""
    sys_encoding = sys.getfilesystemencoding()
    hostname = socket.gethostname()
    if isinstance(hostname, compat.binary_type):
        return hostname.decode(sys_encoding)
    return hostname


def get_old_token_name(app_name):
    """Build the token name (old style). Return an unicode."""
    quoted_app_name = quote(app_name)
    computer_name = gethostname()
    quoted_computer_name = quote(computer_name)

    assert isinstance(computer_name, compat.text_type)
    assert isinstance(quoted_computer_name, compat.text_type)

    return "%s - %s" % (quoted_app_name, quoted_computer_name)


def get_token_name(app_name):
    """Build the token name.. Return an unicode."""
    computer_name = gethostname()
    computer_name = computer_name.replace(TOKEN_SEPARATOR,
                                          SEPARATOR_REPLACEMENT)

    assert isinstance(computer_name, compat.text_type)
    assert isinstance(computer_name, compat.text_type)

    return TOKEN_SEPARATOR.join((app_name, computer_name))


@inlineCallbacks
def try_old_credentials(app_name):
    """Try to get old U1 credentials and format them as new."""
    logger.debug('trying to get old credentials.')
    old_creds = yield UbuntuOneOAuthKeyring().get_credentials(U1_KEY_NAME)
    if old_creds is not None:
        # Old creds found, build a new credentials dict with them
        creds = {
            'consumer_key': "ubuntuone",
            'consumer_secret': "hammertime",
            'name': U1_KEY_NAME,
            'token': old_creds["oauth_token"],
            'token_secret': old_creds["oauth_token_secret"],
        }
        logger.debug('found old credentials')
        returnValue(creds)
    logger.debug('try_old_credentials: No old credentials for this app.')
    returnValue(None)


Keyring = None
if sys.platform in ('win32', 'darwin'):
    from ubuntu_kylin_sso.keyring import pykeyring
    Keyring = pykeyring.Keyring
else:
    from ubuntu_kylin_sso.keyring import linux as linux_kr
    Keyring = linux_kr.Keyring


class UbuntuOneOAuthKeyring(Keyring):
    """A particular Keyring for Ubuntu One."""

    def _get_keyring_attr(self, app_name):
        """Build the keyring attributes for this credentials."""
        return U1_KEY_ATTR



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Tests for the expander widget."""

from PyQt4.QtGui import QLabel
from twisted.internet import defer
from twisted.trial.unittest import TestCase

from ubuntu_kylin_sso.qt.arrow import QArrow
from ubuntu_kylin_sso.qt.expander import QExpanderLabel, QExpander

# pylint: disable=C0103, W0201, W0212


class QExpanderLabelTestCase(TestCase):
    """Test the expander labe."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set tests."""
        yield super(QExpanderLabelTestCase, self).setUp()
        self.label_content = 'test'
        self.label = QExpanderLabel(self.label_content)

    def test_mouse_press_event_arrow_right(self):
        """Test the actions taken when we press the mouse."""
        self.label.arrow.direction = QArrow.RIGHT
        self.label.mousePressEvent(None)
        self.assertEqual(QArrow.DOWN, self.label.arrow.direction)

    def test_mouse_press_event_arrow_down(self):
        """Test the actions taken when we press the mouse."""
        self.label.arrow.direction = QArrow.DOWN
        self.label.mousePressEvent(None)
        self.assertEqual(QArrow.RIGHT, self.label.arrow.direction)

    def test_text(self):
        """Test getting the text."""
        self.assertEqual(self.label.label.text(), self.label.text())

    def test_set_text(self):
        """Test setting the text."""
        new_label = 'new'
        self.label.setText(new_label)
        self.assertEqual(self.label.label.text(), new_label)


class QExpanderTestCase(TestCase):
    """Test the QExpander widget."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set up tests."""
        yield super(QExpanderTestCase, self).setUp()
        self.label_content = 'test'
        self.expander = QExpander(self.label_content)

    def test_init(self):
        """Test the constructor."""

    def test_add_widget(self):
        """Test adding a widget."""
        widget = QLabel('test')
        widget.setObjectName('test')
        self.expander.addWidget(widget)
        self.assertEqual(widget, self.expander.content)
        self.assertEqual(1, self.expander.layout.indexOf(widget))

    def test_add_widget_present(self):
        """Test adding a widge when one is already present."""
        old_widget = QLabel('old')
        widget = QLabel('test')
        self.expander.addWidget(old_widget)
        self.expander.addWidget(widget)
        self.assertEqual(-1, self.expander.layout.indexOf(old_widget))
        self.assertEqual(1, self.expander.layout.indexOf(widget))

    def test_add_widget_expanded(self):
        """Test adding a widget when we are expanded."""
        widget = QLabel('test')
        self.expander._expanded = True
        self.expander.addWidget(widget)
        self.assertEqual(1, self.expander.layout.indexOf(widget))

    def test_add_widget_collapsed(self):
        """Test adding a widget when we are collapsed."""
        widget = QLabel('test')
        self.expander._expanded = False
        self.expander.addWidget(widget)
        self.assertEqual(1, self.expander.layout.indexOf(widget))
        self.assertFalse(widget.isVisible())

    def test_text(self):
        """Test getting the text."""
        self.assertEqual(self.expander.text(), self.expander.label.text())

    def test_set_text(self):
        """Test setting the text."""
        label_content = 'content'
        self.expander.setText(label_content)
        self.assertEqual(label_content, self.expander.label.text())

    def test_expanded(self):
        """Test getting expanded."""
        self.expander._expanded = True
        self.assertEqual(self.expander.expanded(), self.expander._expanded)



# -*- coding: utf-8 -*-
#
# Copyright 2009-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Main module implementation specific for darwin (OS X).

This module should never import from the multiplatform one (main/__init__.py),
but the other way around. Likewise, this module should *not* have any logic
regarding error processing or decision making about when to send a given
signal.

Also, most of the logging is being made in the main module to avoid
duplication between the different platform implementations.

"""

import os
import os.path

from dirspec import basedir
from twisted.internet import defer

from ubuntu_kylin_sso import BACKEND_EXECUTABLE
from ubuntu_kylin_sso.logger import setup_logging
from ubuntu_kylin_sso.utils import get_bin_cmd
from ubuntu_kylin_sso.utils.ipc import BaseClient
from ubuntu_kylin_sso.main.perspective_broker import (
    SSO_SERVICE_NAME,
    SSOLoginClient,
    CredentialsManagementClient,
    UbuntuSSOProxyBase,
)

# Invalid name for signals that are CamelCase
# pylint: disable=C0103


logger = setup_logging("ubuntu_kylin_sso.main.darwin")
SSO_INSTALL_PATH = 'SSOInstallPath'


def get_sso_domain_socket():
    """Compute the domain socket for the sso ipc."""
    path = os.path.join(basedir.xdg_cache_home, 'sso', 'ipc')
    return path


class DescriptionFactory(object):
    """Factory that provides the server and client descriptions."""

    client_description_pattern = 'unix:path=%s'
    server_description_pattern = 'unix:%s'

    def __init__(self):
        """Create a new instance."""
        self.domain = get_sso_domain_socket()
        self.server = self.server_description_pattern % self.domain
        self.client = self.client_description_pattern % self.domain


class UbuntuSSOProxy(UbuntuSSOProxyBase):
    """Object that exposes the diff referenceable objects."""

    name = SSO_SERVICE_NAME

    @property
    def description(self):
        """Get the description on which the SSO pb is running."""
        return DescriptionFactory()

    @property
    def cmdline(self):
        """Get the command line to activate an executable."""
        return get_bin_cmd(BACKEND_EXECUTABLE)


class UbuntuSSOClient(BaseClient):
    """Base client that provides remote access to the sso API."""

    name = SSO_SERVICE_NAME

    clients = {
        'sso_login': SSOLoginClient,
        'cred_manager': CredentialsManagementClient,
    }

    service_name = UbuntuSSOProxy.name
    service_description = UbuntuSSOProxy.description
    service_cmdline = UbuntuSSOProxy.cmdline


@defer.inlineCallbacks
def get_sso_client():
    """Get a client to access the SSO service."""
    result = UbuntuSSOClient()
    yield result.connect()
    defer.returnValue(result)



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Utility to spawn another program from a mainloop."""

import os
import sys

from twisted.internet import utils

from ubuntu_kylin_sso.logger import setup_logging
from ubuntu_kylin_sso.utils import compat


logger = setup_logging("ubuntu_kylin_sso.utils.runner.tx")

NO_SUCH_FILE_OR_DIR = 'OSError: [Errno 2]'


EXE_EXT = ''
if sys.platform == 'win32':
    EXE_EXT = '.exe'


def spawn_program(args, reply_handler, error_handler):
    """Spawn the program specified by 'args' using the twisted reactor.

    When the program finishes, 'reply_handler' will be called with a single
    argument that will be the porgram status code.

    If there is an error, error_handler will be called with an instance of
    SpawnError.

    """

    def child_watch(args):
        """Handle child termination."""
        stdout, stderr, exit_code = args
        if stdout:
            logger.debug('Returned stdout is (exit code was %r): %r',
                         exit_code, stdout)
        if stderr:
            logger.warning('Returned stderr is (exit code was %r): %r',
                           exit_code, stderr)

        if OSError.__name__ in stderr:
            failed_to_start = NO_SUCH_FILE_OR_DIR in stderr
            error_handler(msg=stderr, failed_to_start=failed_to_start)
        else:
            reply_handler(exit_code)

    def handle_error(failure):
        """Handle error when spawning the process."""
        error_handler(msg=failure.getErrorMessage())

    args = list(args)
    program = args[0]
    argv = args[1:]

    bytes_args = []
    for arg in argv:
        if isinstance(arg, compat.text_type):
            arg = arg.encode('utf-8')
        if not isinstance(arg, compat.basestring):
            arg = compat.binary_type(arg)
        bytes_args.append(arg)

    if program and not os.access(program, os.X_OK):
        # handle searching the executable in the PATH, since
        # twisted will not solve that for us :-/
        paths = os.environ['PATH'].split(os.pathsep)
        for path in paths:
            target = os.path.join(path, program)
            if not target.endswith(EXE_EXT):
                target += EXE_EXT
            if os.access(target, os.X_OK):
                program = target
                break

    try:
        d = utils.getProcessOutputAndValue(program, bytes_args, env=os.environ)
    except OSError as e:
        error_handler(msg=e, failed_to_start=True)
    except Exception as e:
        error_handler(msg=e, failed_to_start=False)
    else:
        logger.debug('Spawning the program %r with the twisted reactor '
                     '(returned deferred is %r).', repr(args), d)
        d.addCallback(child_watch)
        d.addErrback(handle_error)



# -*- coding: utf-8 -*-

# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""The test suite for Network Detection UI."""

from PyQt4 import QtGui

from ubuntu_kylin_sso.qt import network_detection_page
from ubuntu_kylin_sso.qt.tests import (
    FakeSignal,
    PageBaseTestCase,
    FakeWizardButtonStyle,
)

# is ok to access private method/attrs in tests
# pylint: disable=W0212, E1101


class NetworkDetectionTestCase(PageBaseTestCase):

    """Test the CurrentUserController."""

    ui_class = network_detection_page.NetworkDetectionPage
    ui_wizard_class = FakeWizardButtonStyle

    def test_initialize_page(self):
        """Check Network detection initialize page."""
        self.ui.initializePage()
        self.assertEqual(self.wizard._next_id, -1)
        self.assertTrue(('setButtonLayout', ([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.CustomButton1,
            QtGui.QWizard.CancelButton], {})),
            self.wizard.called)

    def test_initialize_page_button_property(self):
        """Test the Try Again button properties."""
        self.ui.initializePage()
        self.assertTrue(self.ui.btn_try_again.isDefault())

    def test_try_again_with_connection(self):
        """Check try again method with connection."""
        self.patch(network_detection_page.networkstate, 'is_machine_connected',
            lambda: True)
        fake_signal = FakeSignal()
        fake_signal.target = self._set_called
        self.patch(self.ui, 'connectionDetected', fake_signal)
        self.ui.try_again()
        self.assertEqual(self._called, ((), {}))

    def test_try_again_without_connection(self):
        """Check try again method without connection."""
        self.patch(network_detection_page.networkstate, 'is_machine_connected',
            lambda: False)
        fake_signal = FakeSignal()
        fake_signal.target = self._set_called
        self.patch(self.ui, 'connectionDetected', fake_signal)
        self.ui.try_again()
        self.assertFalse(self._called)



# -*- coding: utf-8 -*-
#
# Copyright 2014 National University of Defense Technology(NUDT) & Kylin Ltd
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""SSO Wizard UI."""

import sys

from PyQt4.QtCore import pyqtSignal
from PyQt4.QtGui import (
    QFrame,
    QVBoxLayout,
    QWizard,
)
from twisted.internet import defer

from ubuntu_kylin_sso import (
    networkstate,
    USER_CANCELLATION,
    USER_SUCCESS,
)
from ubuntu_kylin_sso.logger import setup_gui_logging
from ubuntu_kylin_sso.utils import compat
from ubuntu_kylin_sso.qt import PREFERED_UI_SIZE, WINDOW_TITLE
from ubuntu_kylin_sso.qt.current_user_sign_in_page import CurrentUserSignInPage
from ubuntu_kylin_sso.qt.email_verification_page import EmailVerificationPage
from ubuntu_kylin_sso.qt.error_page import ErrorPage
from ubuntu_kylin_sso.qt.forgotten_password_page import ForgottenPasswordPage
from ubuntu_kylin_sso.qt.loadingoverlay import LoadingOverlay
from ubuntu_kylin_sso.qt.network_detection_page import NetworkDetectionPage
from ubuntu_kylin_sso.qt.reset_password_page import ResetPasswordPage
from ubuntu_kylin_sso.qt.setup_account_page import SetupAccountPage
from ubuntu_kylin_sso.qt.success_page import SuccessPage
from ubuntu_kylin_sso.utils.ui import FINISH

logger = setup_gui_logging('ubuntu_kylin_sso.gui')


class UbuntuSSOWizard(QWizard):
    """Wizard used to create or use sso."""

    # definition of the signals raised by the widget
    recoverableError = pyqtSignal('QString', 'QString')
    loginSuccess = pyqtSignal('QString', 'QString')
    registrationSuccess = pyqtSignal('QString', 'QString')

    def __init__(self, app_name, **kwargs):
        """Create a new wizard."""
        logger.debug('UbuntuSSOWizard: app_name %r, kwargs %r.',
                     app_name, kwargs)
        parent = kwargs.pop('parent', None)
        super(UbuntuSSOWizard, self).__init__(parent=parent)
        self.overlay = LoadingOverlay(self)
        self.overlay.hide()
        self._next_id = -1
        self.setOption(QWizard.HaveFinishButtonOnEarlyPages, True)
        self.setOption(QWizard.NoBackButtonOnStartPage, True)
        self.exit_code = USER_CANCELLATION

        self.app_name = app_name
        self.login_only = kwargs.pop('login_only', False)
        self.close_callback = kwargs.pop('close_callback', lambda: None)

        # store the ids of the pages so that it is easier to access them later
        self._pages = {}

        # prepare kwargs to be suitable for the pages
        kwargs['app_name'] = self.app_name
        kwargs['parent'] = self

        self.network_page = NetworkDetectionPage(self.app_name)
        self.network_page.connectionDetected.connect(self._connection_detected)
        self.addPage(self.network_page)

        # set the diff pages of the QWizard
        self.setup_account = SetupAccountPage(**kwargs)
        self.setup_account.userRegistered.connect(
            self._move_to_email_verification_page)
        # There are no tests for signal connections on
        # this file (LP:1046886)
        self.setup_account.signIn.connect(self._move_to_login_page)
        self.addPage(self.setup_account)

        self.current_user = CurrentUserSignInPage(**kwargs)
        self.current_user.userNotValidated.connect(
            self._move_to_email_verification_page)
        self.current_user.userLoggedIn.connect(self._move_to_success_page)
        self.current_user.passwordForgotten.connect(
            self._move_to_forgotten_page)
        self.current_user.createAccount.connect(
            self._move_to_setup_account_page)
        self.addPage(self.current_user)

        self.email_verification = EmailVerificationPage(**kwargs)
        self.email_verification.registrationSuccess.connect(
            self._move_to_success_page)
        self.addPage(self.email_verification)

        self.success = SuccessPage(**kwargs)
        self.addPage(self.success)

        self.error = ErrorPage(**kwargs)
        self.addPage(self.error)

        self.forgotten = ForgottenPasswordPage(**kwargs)
        self.forgotten.passwordResetTokenSent.connect(
            self._move_to_reset_password_page)
        self.addPage(self.forgotten)

        self.reset_password = ResetPasswordPage(**kwargs)
        back = lambda *a: self._go_back_to_page(self.current_user)
        self.reset_password.passwordChanged.connect(back)
        self.addPage(self.reset_password)

        # set the buttons layout to only have cancel and back since the next
        # buttons are the ones used in the diff pages.
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.BackButton)
        buttons_layout.append(QWizard.CancelButton)
        self.setButtonLayout(buttons_layout)
        self.setWindowTitle(self.app_name)
        self.setWizardStyle(QWizard.ModernStyle)
        self.button(QWizard.CancelButton).clicked.connect(self.close)
        self.setMinimumSize(PREFERED_UI_SIZE['width'],
            PREFERED_UI_SIZE['height'])

        # This is missing tests (LP:1046887)
        if self.login_only:
            self._move_to_login_page()
        else:
            self._move_to_setup_account_page()

    @defer.inlineCallbacks
    def check_network_connection(self):
        """Check if the NetworkDetectionPage is needed to be shown."""
        d = yield networkstate.is_machine_connected()
        if d:
            self._connection_detected()

    def _connection_detected(self):
        """Connection restablished, move to the proper page."""
        if self.login_only:
            self._next_id = self.current_user_page_id
        else:
            self._next_id = self.setup_account_page_id
        self.next()
        self._next_id = -1

    # pylint: disable=C0103

    def showEvent(self, event):
        """Check the network connection before the ui is shown."""
        super(UbuntuSSOWizard, self).showEvent(event)
        self.check_network_connection()

    def nextId(self):
        """Return the id of the next page."""
        return self._next_id

    def addPage(self, page):
        """Add 'page' to this wizard."""
        page_id = super(UbuntuSSOWizard, self).addPage(page)
        page.processingStarted.connect(self.overlay.show)
        page.processingFinished.connect(self.overlay.hide)
        self._pages[page] = page_id

    # pylint: enable=C0103

    def _go_back_to_page(self, page):
        """Move back until it reaches the 'page'."""
        logger.debug('Moving back from page: %s, to page: %s',
            self.currentPage(), page)
        page_id = self._pages[page]
        visited_pages = self.visitedPages()
        for index in reversed(visited_pages):
            if index == page_id:
                break
            self.back()

    def _move_to_reset_password_page(self):
        """Move to the reset password page wizard."""
        logger.debug('Moving to ResetPasswordPage from: %s',
            self.currentPage())
        self._next_id = self.reset_password_page_id
        self.next()
        self._next_id = -1

    def _move_to_email_verification_page(self, email):
        """Move to the email verification page wizard."""
        logger.debug('Moving to EmailVerificationPage from: %s',
            self.currentPage())
        self._next_id = self.email_verification_page_id
        self.email_verification.email = compat.text_type(email)
        self.email_verification.password = self.currentPage().password
        self.next()
        self._next_id = -1

    def _move_to_setup_account_page(self):
        """Move to the setup account page wizard."""
        logger.debug('Moving to SetupAccountPage from: %s',
            self.currentPage())
        self.setStartId(self.setup_account_page_id)
        self.restart()

    def _move_to_login_page(self):
        """Move to the login page wizard."""
        logger.debug('Moving to CurrentUserSignInPage from: %s',
            self.currentPage())
        self.setStartId(self.current_user_page_id)
        self.restart()

    def _move_to_success_page(self):
        """Move to the success page wizard."""
        logger.debug('Moving to SuccessPage from: %s',
            self.currentPage())
        self._next_id = self.success_page_id
        self.next()
        self.setButtonLayout([
            QWizard.Stretch,
            QWizard.FinishButton])

        # The "Finish" button of the wizard has not been translated. Do it manually.
        self.setButtonText(QWizard.FinishButton, FINISH)

        self.button(QWizard.FinishButton).setEnabled(True)
        self.button(QWizard.FinishButton).setFocus()
        self.exit_code = USER_SUCCESS
        self._next_id = -1

    def _move_to_forgotten_page(self):
        """Move to the forgotten page wizard."""
        logger.debug('Moving to ForgottenPasswordPage from: %s',
            self.currentPage())
        self._next_id = self.forgotten_password_page_id
        self.next()
        self._next_id = -1

    @property
    def setup_account_page_id(self):
        """Return the id of the page used for sign in."""
        return self._pages[self.setup_account]

    @property
    def email_verification_page_id(self):
        """Return the id of the verification page."""
        return self._pages[self.email_verification]

    @property
    def current_user_page_id(self):
        """Return the id used to signin by a current user."""
        return self._pages[self.current_user]

    @property
    def success_page_id(self):
        """Return the id of the success page."""
        return self._pages[self.success]

    @property
    def forgotten_password_page_id(self):
        """Return the id of the forgotten password page."""
        return self._pages[self.forgotten]

    @property
    def reset_password_page_id(self):
        """Return the id of the reset password page."""
        return self._pages[self.reset_password]

    @property
    def error_page_id(self):
        """Return the id of the error page."""
        return self._pages[self.error]

    def done(self, result):
        """Replace the done method from the wizard."""
        self.closeEvent(None)

    # pylint: disable=C0103

    def closeEvent(self, event):
        """Catch close event and send the proper return code."""
        if self.parent() is not None:
            self.parent().close()
        else:
            sys.exit(self.exit_code)

    def resizeEvent(self, event):
        """Resize the overlay to fit all the widget."""
        super(UbuntuSSOWizard, self).resizeEvent(event)
        self.overlay.resize(event.size())

    # pylint: enable=C0103


class UbuntuSSOClientGUI(QFrame):
    """Ubuntu single sign-on GUI."""

    def __init__(self, app_name, **kwargs):
        """Create a new instance."""
        super(UbuntuSSOClientGUI, self).__init__()
        self.setObjectName("ubuntussoframe")
        vbox = QVBoxLayout(self)
        vbox.setContentsMargins(0, 0, 0, 0)
        logger.debug('UbuntuSSOClientGUI: app_name %r, kwargs %r.',
                     app_name, kwargs)
        self.app_name = app_name
        self.setWindowTitle(WINDOW_TITLE)
        # create the controller and the ui, then set the cb and call the show
        # method so that we can work
        self.wizard = UbuntuSSOWizard(app_name=app_name, **kwargs)
        vbox.addWidget(self.wizard)

    # pylint: disable=C0103

    def closeEvent(self, event):
        """Catch close event and send the proper return code."""
        sys.exit(self.wizard.exit_code)

    # pylint: enable=C0103



# -*- coding: utf-8 -*-
#
# Copyright 2011-2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""The common bits of a webclient."""

import collections

from httplib2 import iri2uri
from oauthlib.oauth1 import (
    Client, SIGNATURE_HMAC, SIGNATURE_PLAINTEXT, SIGNATURE_TYPE_AUTH_HEADER,
    SIGNATURE_TYPE_QUERY)
from twisted.internet import defer

from ubuntu_kylin_sso import (USER_SUCCESS,
                        UI_PROXY_CREDS_DIALOG,
                        UI_SSL_DIALOG)
from ubuntu_kylin_sso.logger import setup_logging
from ubuntu_kylin_sso.utils.runner import spawn_program
from ubuntu_kylin_sso.utils.ui import SSL_DETAILS_TEMPLATE
from ubuntu_kylin_sso.utils.webclient.timestamp import TimestampChecker


logger = setup_logging("ubuntu_kylin_sso.utils.webclient.common")


class WebClientError(Exception):
    """An http error happened while calling the webservice."""


class UnauthorizedError(WebClientError):
    """The request ended with bad_request, unauthorized or forbidden."""


class ProxyUnauthorizedError(WebClientError):
    """Failure raised when there is an issue with the proxy auth."""


class Response(object):
    """A response object."""

    def __init__(self, content, headers=None):
        """Initialize this instance."""
        self.content = content
        self.headers = headers


class HeaderDict(collections.defaultdict):
    """A case insensitive dict for headers."""

    # pylint: disable=E1002
    def __init__(self, *args, **kwargs):
        """Handle case-insensitive keys."""
        super(HeaderDict, self).__init__(list, *args, **kwargs)
        # pylint: disable=E1101
        for key, value in self.items():
            super(HeaderDict, self).__delitem__(key)
            self[key] = value

    def __setitem__(self, key, value):
        """Set the value with a case-insensitive key."""
        super(HeaderDict, self).__setitem__(key.lower(), value)

    def __getitem__(self, key):
        """Get the value with a case-insensitive key."""
        return super(HeaderDict, self).__getitem__(key.lower())

    def __delitem__(self, key):
        """Delete the item with the case-insensitive key."""
        super(HeaderDict, self).__delitem__(key.lower())

    def __contains__(self, key):
        """Check the containment with a case-insensitive key."""
        return super(HeaderDict, self).__contains__(key.lower())


class BaseWebClient(object):
    """The webclient base class, to be extended by backends."""

    timestamp_checker = None

    def __init__(self, appname='', username=None, password=None,
                 oauth_sign_plain=False):
        """Initialize this instance."""
        self.appname = appname
        self.username = username
        self.password = password
        self.proxy_username = None
        self.proxy_password = None
        self.oauth_sign_plain = oauth_sign_plain

    def request(self, iri, method="GET", extra_headers=None,
                oauth_credentials=None, post_content=None):
        """Return a deferred that will be fired with a Response object."""
        raise NotImplementedError

    @classmethod
    def get_timestamp_checker(cls):
        """Get the timestamp checker for this class of webclient."""
        if cls.timestamp_checker is None:
            cls.timestamp_checker = TimestampChecker(cls)
        return cls.timestamp_checker

    def get_timestamp(self):
        """Get a timestamp synchronized with the server."""
        return self.get_timestamp_checker().get_faithful_time()

    def force_use_proxy(self, settings):
        """Setup this webclient to use the given proxy settings."""
        raise NotImplementedError

    def iri_to_uri(self, iri):
        """Transform a unicode iri into a ascii uri."""
        if not isinstance(iri, unicode):
            raise TypeError('iri %r should be unicode.' % iri)
        return bytes(iri2uri(iri))

    def build_oauth_request(self, method, uri, credentials, timestamp,
                            parameters=None, as_query=True):
        """Build an oauth request given some credentials."""

        # oauthlib is requiring the timestamp to be a string, because
        # it tries to escape all the parameters.
        oauth_client = Client(credentials['consumer_key'],
                              credentials['consumer_secret'],
                              credentials['token'],
                              credentials['token_secret'],
                              signature_method=(SIGNATURE_PLAINTEXT
                                                if self.oauth_sign_plain
                                                else SIGNATURE_HMAC),
                              signature_type=(SIGNATURE_TYPE_QUERY
                                              if as_query
                                              else SIGNATURE_TYPE_AUTH_HEADER),
                              timestamp=str(timestamp))

        try:
            url, signed_headers, body = oauth_client.sign(
                uri, method, parameters if parameters is not None else {},
                {'Content-Type': 'application/x-www-form-urlencoded'})
        except ValueError:
            url, signed_headers, body = oauth_client.sign(uri, method)

        return url, signed_headers, body

    @defer.inlineCallbacks
    def build_request_headers(self, uri, method="GET", extra_headers=None,
                              oauth_credentials=None):
        """Build the headers for a request."""
        if extra_headers:
            headers = dict(extra_headers)
        else:
            headers = {}

        if oauth_credentials:
            timestamp = yield self.get_timestamp()
            url, signed_headers, body = self.build_oauth_request(
                method, uri, oauth_credentials, timestamp,
                as_query=False)
            headers.update(signed_headers)

        defer.returnValue(headers)

    @defer.inlineCallbacks
    def build_signed_iri(self, iri, credentials, parameters=None):
        """Build a new iri signing 'iri' with 'credentials'."""
        uri = self.iri_to_uri(iri)
        timestamp = yield self.get_timestamp()
        url, signed_headers, body = self.build_oauth_request(
            method='GET', uri=uri, credentials=credentials,
            timestamp=timestamp, parameters=parameters)
        defer.returnValue(url)

    def shutdown(self):
        """Shut down all pending requests (if possible)."""

    @defer.inlineCallbacks
    def _load_proxy_creds_from_keyring(self, domain):
        """Load the proxy creds from the keyring."""
        from ubuntu_kylin_sso.keyring import Keyring
        keyring = Keyring()
        try:
            creds = yield keyring.get_credentials(str(domain))
            logger.debug('Got credentials from keyring.')
        except Exception as e:
            logger.error('Error when retrieving the creds.')
            raise WebClientError('Error when retrieving the creds.', e)
        if creds is not None:
            # if we are loading the same creds it means that we got the wrong
            # ones
            if (self.proxy_username == creds['username'] and
                    self.proxy_password == creds['password']):
                defer.returnValue(False)
            else:
                self.proxy_username = creds['username']
                self.proxy_password = creds['password']
                defer.returnValue(True)
        logger.debug('Proxy creds not in keyring.')
        defer.returnValue(False)

    def _launch_proxy_creds_dialog(self, domain, retry):
        """Launch the dialog used to get the creds."""
        from ubuntu_kylin_sso.utils import get_bin_cmd
        args = get_bin_cmd(UI_PROXY_CREDS_DIALOG)

        args += ['--domain', domain]
        if retry:
            args += ['--retry']
        return spawn_program(args)

    @defer.inlineCallbacks
    def request_proxy_auth_credentials(self, domain, retry):
        """Request the auth creds to the user."""
        if not retry:
            if (self.proxy_username is not None
                    and self.proxy_password is not None):
                logger.debug('Not retry and credentials are present.')
                defer.returnValue(True)
            else:
                creds_loaded = yield self._load_proxy_creds_from_keyring(
                    domain)
                if creds_loaded:
                    defer.returnValue(True)

        try:
            return_code = yield self._launch_proxy_creds_dialog(domain, retry)
        except Exception as e:
            logger.error('Error when running external ui process.')
            raise WebClientError('Error when running external ui process.', e)

        if return_code == USER_SUCCESS:
            creds_loaded = yield self._load_proxy_creds_from_keyring(domain)
            defer.returnValue(creds_loaded)
        else:
            logger.debug('Could not retrieve the credentials. Return code: %r',
                         return_code)
            defer.returnValue(False)

    def format_ssl_details(self, details):
        """Return a formatted string with the details."""
        return SSL_DETAILS_TEMPLATE % details

    def _launch_ssl_dialog(self, domain, details):
        """Launch a dialog used to approve the ssl cert."""
        from ubuntu_kylin_sso.utils import get_bin_cmd

        args = get_bin_cmd(UI_SSL_DIALOG)
        args += ['--domain', domain,
                 '--details', details,
                 '--appname', self.appname]

        return spawn_program(args)

    def _was_ssl_accepted(self, cert_details):
        """Return if the cert was already accepted."""
        # TODO: Ensure that we look at pinned certs in a following branch
        return False

    @defer.inlineCallbacks
    def request_ssl_cert_approval(self, domain, details):
        """Request the user for ssl approval."""
        if self._was_ssl_accepted(details):
            defer.returnValue(True)

        return_code = yield self._launch_ssl_dialog(domain, details)
        defer.returnValue(return_code == USER_SUCCESS)



# -*- coding: utf-8 -*-

#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Tests for the tcpactivation module."""

from __future__ import print_function

import twisted.internet
from twisted.internet import defer, protocol, task
from twisted.trial.unittest import TestCase

from ubuntuone.devtools.testcases.txsocketserver import (
    ServerTestCase,
    TidyTCPServer,
)

from ubuntu_kylin_sso.utils import tcpactivation
from ubuntu_kylin_sso.utils.tcpactivation import (
    ActivationClient,
    ActivationConfig,
    ActivationDetector,
    ActivationInstance,
    ActivationTimeoutError,
    AlreadyStartedError,
    NullProtocol,
    PortDetectFactory,
)

SAMPLE_SERVICE = "test_service_name"
SAMPLE_CMDLINE = ["python", __file__, "-server"]
SAMPLE_SERVER_DESCRIPTION = 'tcp:55555:interface=127.0.0.1'
SAMPLE_CLIENT_DESCRIPTION = 'tcp:host=127.0.0.1:port=55555'


class FakeServerProtocol(protocol.Protocol):
    """A test protocol."""

    def dataReceived(self, data):
        """Echo the data received."""
        self.transport.write(data)


class FakeServerFactory(protocol.ServerFactory):
    """A factory for the test server."""

    protocol = FakeServerProtocol


class FakeTransport(object):
    """A fake transport."""

    connectionLost = False

    def loseConnection(self):
        """Remember that the connection was dropped."""
        self.connectionLost = True


class FakeDescriptionFactory(object):
    """A fake description factory."""

    def __init__(self, server_description, client_description):
        """Create a new instace."""
        self.server = server_description
        self.client = client_description


class AsyncSleepTestCase(TestCase):
    """Tests for the async_sleep function."""

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this test instance."""
        yield super(AsyncSleepTestCase, self).setUp()
        self.test_timeout = 5.0
        self.clock = task.Clock()
        self.patch(twisted.internet, "reactor", self.clock)
        self.d = tcpactivation.async_sleep(self.test_timeout)

    def test_async_sleep_not_fired_immediately(self):
        """The async_sleep deferred is not fired immediately."""
        self.assertFalse(self.d.called, "Must not be fired immediately.")

    def test_async_sleep_not_fired_in_a_bit(self):
        """The async_sleep deferred is not fired before the right time."""
        self.clock.advance(self.test_timeout / 2)
        self.assertFalse(self.d.called, "Must not be fired yet.")

    def test_async_sleep_fired_at_the_right_time(self):
        """The async_sleep deferred is fired at the right time."""
        self.clock.advance(self.test_timeout)
        self.assertTrue(self.d.called, "Must be fired by now.")


class NullProtocolTestCase(TestCase):
    """A test for the NullProtocol class."""

    def test_drops_connection(self):
        """The protocol drops the connection."""
        np = NullProtocol()
        np.transport = FakeTransport()
        np.connectionMade()
        self.assertTrue(np.transport.connectionLost,
                        "the connection must be dropped.")


class PortDetectFactoryTestCase(TestCase):
    """Tests for the PortDetectFactory."""

    timeout = 2

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this test instance."""
        yield super(PortDetectFactoryTestCase, self).setUp()
        self.factory = PortDetectFactory()

    @defer.inlineCallbacks
    def test_is_listening(self):
        """Test that the deferred returns True when something is listening."""
        addr = (tcpactivation.LOCALHOST, SAMPLE_CLIENT_DESCRIPTION)
        self.factory.buildProtocol(addr)
        is_listening = yield self.factory.is_listening()
        self.assertTrue(is_listening)

    @defer.inlineCallbacks
    def test_connection_lost(self):
        """Test that the deferred returns False when the connection is lost."""
        self.factory.clientConnectionLost(None, "test reason")
        is_listening = yield self.factory.is_listening()
        self.assertFalse(is_listening)

    @defer.inlineCallbacks
    def test_connection_failed(self):
        """Test that the deferred returns False when the connection fails."""
        self.factory.clientConnectionFailed(None, "test reason")
        is_listening = yield self.factory.is_listening()
        self.assertFalse(is_listening)

    @defer.inlineCallbacks
    def test_connection_failed_then_lost(self):
        """It's not an error if two events happen."""
        self.factory.clientConnectionFailed(None, "test reason")
        self.factory.clientConnectionLost(None, "test reason")
        is_listening = yield self.factory.is_listening()
        self.assertFalse(is_listening)

    @defer.inlineCallbacks
    def test_connection_works_then_lost(self):
        """It's not an error if two events happen."""
        addr = (tcpactivation.LOCALHOST, SAMPLE_CLIENT_DESCRIPTION)
        self.factory.buildProtocol(addr)
        d = self.factory.is_listening()
        self.factory.clientConnectionLost(None, "test reason")
        is_listening = yield d
        self.assertTrue(is_listening)


class ActivationConfigTestCase(TestCase):
    """Tests for the ActivationConfig class."""

    def test_initialization(self):
        """Test the constructor."""
        config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE,
                                                   SAMPLE_CLIENT_DESCRIPTION)
        self.assertEqual(config.service_name, SAMPLE_SERVICE)
        self.assertEqual(config.command_line, SAMPLE_CMDLINE)
        self.assertEqual(config.description, SAMPLE_CLIENT_DESCRIPTION)


class ActivationDetectorTestCase(TestCase):
    """Tests for the ActivationDetector class."""

    timeoue = 3

    client_description = 'tcp:host=127.0.0.1:port=55555'
    server_description = 'tcp:55555:interface=127.0.0.1'

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this test instance."""
        yield super(ActivationDetectorTestCase, self).setUp()
        self.description_factory = FakeDescriptionFactory(
            self.server_description,
            self.client_description)
        self.config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE,
                                       self.description_factory)

    def get_server(self):
        """Return the server to be used to run the tests."""
        return TidyTCPServer()

    def test_initialization(self):
        """Test the constructor."""
        self.config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE,
                                       self.description_factory)
        ai = ActivationDetector(self.config)
        self.assertEqual(ai.config, self.config)

    @defer.inlineCallbacks
    def test_is_not_already_running(self):
        """Test the is_already_running method returns False."""
        ad = ActivationDetector(self.config)
        result = yield ad.is_already_running()
        self.assertFalse(result, "It should not be already running.")

    @defer.inlineCallbacks
    def test_is_already_running(self):
        """The is_already_running method returns True if already started."""
        server = self.get_server()
        self.addCleanup(server.clean_up)

        class TestConnect(object):
            """A fake connection object."""

            @defer.inlineCallbacks
            def connect(my_self, factory):
                """A fake connection."""

                connected_factory = yield server.connect_client(
                        PortDetectFactory)
                self.patch(factory, 'is_listening',
                        connected_factory.is_listening)
                defer.returnValue(connected_factory)

        self.patch(tcpactivation, 'clientFromString',
                lambda *args: TestConnect())

        yield server.listen_server(protocol.ServerFactory)

        ad = ActivationDetector(self.config)
        result = yield ad.is_already_running()
        self.assertTrue(result, "It should be already running.")


class ActivationClientTestCase(TestCase):
    """Tests for the ActivationClient class."""

    timeout = 2

    server_description = 'tcp:55555:interface=127.0.0.1'
    client_description = 'tcp:host=127.0.0.1:port=55555'

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this test instance."""
        yield super(ActivationClientTestCase, self).setUp()
        self.description_factory = FakeDescriptionFactory(
            self.server_description,
            self.client_description)
        self.config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE,
                                       self.description_factory)

    def test_initialization(self):
        """Test the constructor."""
        ac = ActivationClient(self.config)
        self.assertEqual(ac.config, self.config)

    @defer.inlineCallbacks
    def test_do_get_active_description_running(self):
        """Test the _do_get_active_description method when is running."""
        ac = ActivationClient(self.config)
        self.patch(ac, "is_already_running", lambda: defer.succeed(True))
        result = yield ac._do_get_active_description()
        self.assertEqual(result, self.client_description)

    @defer.inlineCallbacks
    def test_do_get_active_description_not_running(self):
        """Test _do_get_active_description method when is not running."""
        server_spawned = []
        ac = ActivationClient(self.config)
        self.patch(ac, "_spawn_server",
                                lambda *args: server_spawned.append(args))
        self.patch(ac, "is_already_running", lambda: defer.succeed(False))
        self.patch(ac, "_wait_server_active", lambda: defer.succeed(None))
        result = yield ac._do_get_active_description()
        self.assertEqual(result, self.client_description)
        self.assertEqual(len(server_spawned), 1)

    def test_get_active_description_waits_classwide(self):
        """Test the get_active_description method locks classwide."""
        d = defer.Deferred()
        ac1 = ActivationClient(self.config)
        ac2 = ActivationClient(self.config)
        self.patch(ac1, "_do_get_active_description", lambda: d)
        self.patch(ac2, "_do_get_active_description",
                   lambda: defer.succeed(None))
        ac1.get_active_client_description()
        d2 = ac2.get_active_client_description()
        self.assertFalse(d2.called, "The second must wait for the first.")
        d.callback(self.client_description)
        self.assertTrue(d2.called, "The second can fire after the first.")

    def test_wait_server_active(self):
        """Test the _wait_server_active method."""
        ac = ActivationClient(self.config)
        clock = task.Clock()
        self.patch(twisted.internet, "reactor", clock)
        self.patch(ac, "is_already_running", lambda: defer.succeed(False))

        d = ac._wait_server_active()

        self.assertFalse(d.called, "The deferred should not be fired yet.")
        clock.advance(tcpactivation.DELAY_BETWEEN_CHECKS)
        self.assertFalse(d.called, "The deferred should not be fired yet.")
        self.patch(ac, "is_already_running", lambda: defer.succeed(True))
        clock.advance(tcpactivation.DELAY_BETWEEN_CHECKS)
        self.assertTrue(d.called, "The deferred should be fired by now.")

    def test_wait_server_timeouts(self):
        """If the server takes too long to start then timeout."""
        ac = ActivationClient(self.config)
        clock = task.Clock()
        self.patch(twisted.internet, "reactor", clock)
        self.patch(ac, "is_already_running", lambda: defer.succeed(False))
        d = ac._wait_server_active()
        clock.pump([tcpactivation.DELAY_BETWEEN_CHECKS] *
                      tcpactivation.NUMBER_OF_CHECKS)
        return self.assertFailure(d, ActivationTimeoutError)

    def test_spawn_server(self):
        """Test the _spawn_server method."""
        popen_calls = []
        ac = ActivationClient(self.config)
        self.patch(tcpactivation.subprocess, "Popen",
                   lambda *args, **kwargs: popen_calls.append((args, kwargs)))
        ac._spawn_server()
        self.assertEqual(len(popen_calls), 1)


class ActivationInstanceTestCase(ServerTestCase):
    """Tests for the ActivationServer class."""

    timeout = 2

    server_description = 'tcp:55555:interface=127.0.0.1'
    client_description = 'tcp:host=127.0.0.1:port=55555'

    @defer.inlineCallbacks
    def setUp(self):
        """Initialize this test instance."""
        yield super(ActivationInstanceTestCase, self).setUp()
        self.description_factory = FakeDescriptionFactory(
            self.server_description,
            self.client_description)
        self.config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE,
                                       self.description_factory)

    def get_server(self):
        """Return the server to be used to run the tests."""
        return TidyTCPServer()

    def test_initialization(self):
        """Test the constructor."""
        ai = ActivationInstance(self.config)
        self.assertEqual(ai.config, self.config)

    @defer.inlineCallbacks
    def test_get_sever_description(self):
        """Test the get_description method."""
        ai = ActivationInstance(self.config)
        self.patch(ai, 'is_already_running', lambda: False)

        description = yield ai.get_server_description()
        self.assertEqual(description, self.server_description)

    @defer.inlineCallbacks
    def test_get_description_fails_if_service_already_started(self):
        """The get_description method fails if service already started."""
        server = self.get_server()
        self.addCleanup(server.clean_up)

        class TestConnect(object):
            """A fake connection object."""

            @defer.inlineCallbacks
            def connect(my_self, factory):
                """A fake connection."""

                connected_factory = yield server.connect_client(
                        PortDetectFactory)
                self.patch(factory, 'is_listening',
                        connected_factory.is_listening)
                defer.returnValue(connected_factory)

        self.patch(tcpactivation, 'clientFromString',
                lambda *args: TestConnect())

        yield server.listen_server(protocol.ServerFactory)

        ai = ActivationInstance(self.config)
        yield self.assertFailure(ai.get_server_description(),
                                 AlreadyStartedError)


def server_test(config):
    """An IRL test of the server."""
    from twisted.internet import reactor

    def got_description(description):
        """The description was found."""
        print("got server description:", description)

        # start listening
        f = FakeServerFactory()
        reactor.listenTCP(description, f)

        # try to get the description again
        get_description()

    def already_started(failure):
        """This instance was already started."""
        print("already started!")
        reactor.callLater(3, reactor.stop)

    def get_description():
        """Try to get the description number."""
        get_description_d = ai.get_server_description()
        get_description_d.addCallback(got_description)
        get_description_d.addErrback(already_started)

    print("starting the server.")
    ai = ActivationInstance(config)
    get_description()
    reactor.run()


def client_test(config):
    """An IRL test of the client."""
    from twisted.internet import reactor
    print("starting the client.")
    ac = ActivationClient(config)
    d = ac.get_active_client_description()

    def got_description(description):
        """The description was found."""
        print("client got server description:", description)
        reactor.stop()

    d.addCallback(got_description)
    reactor.run()


def irl_test():
    """Do an IRL test of the client and the server."""
    import sys
    description_f = FakeDescriptionFactory(SAMPLE_SERVER_DESCRIPTION,
                                           SAMPLE_CLIENT_DESCRIPTION)
    config = ActivationConfig(SAMPLE_SERVICE, SAMPLE_CMDLINE, description_f)
    if "-server" in sys.argv[1:]:
        server_test(config)
    else:
        client_test(config)

if __name__ == "__main__":
    irl_test()



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Qt implementation of the SSL ui."""

import argparse
import sys

from PyQt4.QtGui import (
    QApplication,
    QDialog,
    QScrollArea,
    QStyle,
    QTextEdit,
)

from ubuntu_kylin_sso import USER_CANCELLATION, USER_SUCCESS
from ubuntu_kylin_sso.logger import setup_gui_logging
from ubuntu_kylin_sso.qt.ui import resources_rc
from ubuntu_kylin_sso.qt.expander import QExpander
from ubuntu_kylin_sso.qt.ui.ssl_dialog_ui import Ui_SSLDialog
from ubuntu_kylin_sso.utils.ui import (
    CANCEL_BUTTON,
    SSL_APPNAME_HELP,
    SSL_DETAILS_HELP,
    SSL_DIALOG_TITLE,
    SSL_DESCRIPTION,
    SSL_DOMAIN_HELP,
    SSL_HEADER,
    SSL_EXPLANATION,
    SSL_FIRST_REASON,
    SSL_SECOND_REASON,
    SSL_THIRD_REASON,
    SSL_CERT_DETAILS,
    SSL_NOT_SURE,
    SSL_REMEMBER_DECISION,
    SSL_HELP_BUTTON,
    SSL_CONNECT_BUTTON,
)

REASONS_TEMPLATE = ('<p>%(explanation)s</p>'
                    '<ul style="margin-left:-10px">'
                    '<li style="margin-bottom:5px;">%(first_reason)s</li>'
                    '<li style="margin-bottom:5px;">%(second_reason)s</li>'
                    '<li>%(third_reason)s</li>'
                    '</ul>')

logger = setup_gui_logging("ubuntu_kylin_sso.qt.proxy_dialog")
assert(resources_rc)


class SSLDialog(QDialog):
    """"Dialog used to show SSL exceptions."""

    def __init__(self, app_name, domain=None, details=None, parent=None):
        """Create a new instance."""
        super(SSLDialog, self).__init__()
        if domain is None:
            logger.debug('Domain passed as None.')
            domain = ''
        self.domain = domain
        if details is None:
            logger.debug('Details passed as None.')
            details = ''
        self.details = details
        self.app_name = app_name
        self.ssl_text = None
        self.expander = None
        self.ui = Ui_SSLDialog()
        self.ui.setupUi(self)
        self.setWindowTitle(SSL_DIALOG_TITLE)
        self._set_expander()
        self._set_labels()
        self._set_buttons()
        self._set_icon()

    def _set_labels(self):
        """Set the labels translations."""
        self.ui.title_label.setText(SSL_HEADER)
        explanation = SSL_EXPLANATION % dict(domain=self.domain)
        intro = REASONS_TEMPLATE % dict(explanation=explanation,
                                        first_reason=SSL_FIRST_REASON,
                                        second_reason=SSL_SECOND_REASON,
                                        third_reason=SSL_THIRD_REASON)
        self.ui.intro_label.setText(intro)
        self.ui.not_sure_label.setText(SSL_NOT_SURE %
                                       {'app_name': self.app_name})
        self.ui.remember_checkbox.setText(SSL_REMEMBER_DECISION)

    def _on_cancel_clicked(self):
        """Cancel was cliked."""
        logger.debug('User canceled the ssl dialog.')
        self.done(USER_CANCELLATION)

    def _on_connect_clicked(self):
        """Connect was clicked."""
        logger.debug('User accepted the ssl certificate.')
        self.done(USER_SUCCESS)

    def _set_buttons(self):
        """Set the labels of the buttons."""
        self.ui.help_button.setText(SSL_HELP_BUTTON)
        self.ui.cancel_button.setText(CANCEL_BUTTON)
        self.ui.cancel_button.clicked.connect(self._on_cancel_clicked)
        self.ui.connect_button.setText(SSL_CONNECT_BUTTON)
        self.ui.connect_button.clicked.connect(self._on_connect_clicked)

    def _set_expander(self):
        """Set the expander widget."""
        self.ssl_text = QTextEdit()
        self.ssl_text.setText(self.details)
        scroll_area = QScrollArea()
        scroll_area.setViewport(self.ssl_text)
        scroll_area.setFixedHeight(50)

        self.expander = QExpander(SSL_CERT_DETAILS)
        self.expander.addWidget(scroll_area)
        self.ui.expander_layout.insertWidget(2, self.expander)

    def _set_icon(self):
        """Set the icon used in the dialog."""
        icon = self.style().standardIcon(QStyle.SP_MessageBoxWarning)
        self.ui.logo_label.setText('')
        self.ui.logo_label.setPixmap(icon.pixmap(48, 48))


def parse_args():
    """Parse sys.arg options."""
    parser = argparse.ArgumentParser(
            description=SSL_DESCRIPTION)
    parser.add_argument('--domain', required=True,
        help=SSL_DOMAIN_HELP)
    parser.add_argument('--details', required=True,
        help=SSL_DETAILS_HELP)
    parser.add_argument('--appname', required=True,
        help=SSL_APPNAME_HELP)
    return parser.parse_args()


def main():
    """Main method used to show the creds dialog."""
    app = QApplication(sys.argv)
    assert(app)
    args = parse_args()
    win = SSLDialog(args.appname, domain=args.domain, details=args.details)
    return_code = win.exec_()
    sys.exit(return_code)



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Page to allow the user to login into Ubuntu Single Sign On."""

from functools import partial

from PyQt4 import QtGui, QtCore

from ubuntu_kylin_sso import NO_OP
from ubuntu_kylin_sso.logger import setup_gui_logging
from ubuntu_kylin_sso.qt import LINK_STYLE, build_general_error_message
from ubuntu_kylin_sso.qt.sso_wizard_page import SSOWizardPage
from ubuntu_kylin_sso.qt.ui.current_user_sign_in_ui import Ui_CurrentUserSignInPage
from ubuntu_kylin_sso.utils import compat
from ubuntu_kylin_sso.utils.ui import (
    CANCEL_BUTTON,
    CREATE_ACCOUNT_LABEL,
    #EMAIL_LABEL,
    EMAIL_USERNAME,
    FORGOTTEN_PASSWORD_BUTTON,
    is_correct_email,
    LOGIN_PASSWORD_LABEL,
    LOGIN_TITLE,
    LOGIN_SUBTITLE,
    SIGN_IN_BUTTON,
)
from ubuntu_kylin_sso.constants import (REGISTER_URL, FORGET_PASSWORD_URL)


logger = setup_gui_logging('ubuntu_kylin_sso.current_user_sign_in_page')


class CurrentUserSignInPage(SSOWizardPage):
    """Wizard Page that lets a current user Sign into Ubuntu Single Sign On."""

    ui_class = Ui_CurrentUserSignInPage
    userLoggedIn = QtCore.pyqtSignal(compat.text_type)
    passwordForgotten = QtCore.pyqtSignal()
    createAccount = QtCore.pyqtSignal()
    userNotValidated = QtCore.pyqtSignal(compat.text_type)

    @property
    def _signals(self):
        """The signals to connect to the backend."""
        result = {'LoggedIn':
                      self._filter_by_app_name(self.on_logged_in),
                  'LoginError':
                      self._filter_by_app_name(self.on_login_error),
                  'UserNotValidated':
                      self._filter_by_app_name(self.on_user_not_validated),
                  }
        return result

    @property
    def password(self):
        """Return the content of the password edit."""
        return compat.text_type(self.ui.password_edit.text())

    def on_user_not_validated(self, app_name, email):
        """Show the validate email page."""
        self.hide_overlay()
        email = compat.text_type(self.ui.email_edit.text())
        self.userNotValidated.emit(email)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def nextId(self):
        """Provide the next id."""
        return self.next

    def initializePage(self):
        """Setup UI details."""
        logger.debug('initializePage - About to show CurrentUserSignInPage')
        self.setButtonText(QtGui.QWizard.CancelButton, CANCEL_BUTTON)
        # Layout without custom button 1,
        # without finish button
        self.wizard().setButtonLayout([])

        # Set sign_in_button as default when the page is shown.
        self.ui.sign_in_button.setDefault(True)
        self.ui.sign_in_button.setEnabled(False)

    def cleanupPage(self):
        """Reset the wizard buttons."""
        super(CurrentUserSignInPage, self).cleanupPage()
        self.wizard().setButtonLayout([QtGui.QWizard.Stretch])

    def _set_translated_strings(self):
        """Set the translated strings."""
        self.setTitle(LOGIN_TITLE.format(app_name=self.app_name))
        self.setSubTitle(LOGIN_SUBTITLE % {'app_name': self.app_name})
        self.ui.email_label.setText(EMAIL_USERNAME)
        self.ui.password_label.setText(LOGIN_PASSWORD_LABEL)
        # Now use UbuntuKylin forum to setup account - 20141015 Robert
        # forgotten_text = LINK_STYLE.format(link_url='#',
        #                     link_text=FORGOTTEN_PASSWORD_BUTTON)
        forgotten_text = LINK_STYLE.format(link_url=FORGET_PASSWORD_URL,
                            link_text=FORGOTTEN_PASSWORD_BUTTON)
        self.ui.forgot_password_label.setText(forgotten_text)
        link_text = CREATE_ACCOUNT_LABEL.format(app_name=self.app_name)
        # Now use UbuntuKylin forum to setup account - 20141015 Robert
        # account_text = LINK_STYLE.format(link_url='#', link_text=link_text)
        account_text = LINK_STYLE.format(link_url=REGISTER_URL, link_text=link_text)
        self.ui.create_account_label.setText(account_text)
        self.ui.sign_in_button.setText(SIGN_IN_BUTTON)

    def _connect_ui(self):
        """Connect the buttons to perform actions."""
        self.ui.forgot_password_label.linkActivated.connect(
                                                    self.on_forgotten_password)
        self.ui.create_account_label.linkActivated.connect(
                                                    self.on_create_account)
        self.ui.email_edit.textChanged.connect(self._validate)
        self.ui.password_edit.textChanged.connect(self._validate)
        self.ui.sign_in_button.clicked.connect(self.login)

    def _validate(self):
        """Perform input validation."""
        # Now sso server accept both email and username login - 20141015 Robert
        # correct_mail = is_correct_email(
        #                 compat.text_type(self.ui.email_edit.text()))
        correct_mail = True
        correct_password = len(
                        compat.text_type(self.ui.password_edit.text())) > 0
        enabled = correct_mail and correct_password
        self.ui.sign_in_button.setEnabled(enabled)

    def login(self):
        """Perform the login using the self.backend."""
        # grab the data from the view and call the backend
        email = compat.text_type(self.ui.email_edit.text())
        logger.info('CurrentUserSignInPage.login for: %s', email)
        password = compat.text_type(self.ui.password_edit.text())
        args = (self.app_name, email, password)
        f = self.backend.login

        self.hide_error()
        self.show_overlay()
        error_handler = partial(self._handle_error, f, self.on_login_error)
        f(*args, reply_handler=NO_OP, error_handler=error_handler)

    def on_login_error(self, app_name, error):
        """There was an error when login in."""
        # let the user know
        logger.error('Got error when login %s, error: %s',
            self.app_name, error)
        self.show_error(build_general_error_message(error))

    def on_logged_in(self, app_name, result):
        """We managed to log in."""
        logger.info('Logged in for %s', app_name)
        self.hide_overlay()
        email = compat.text_type(self.ui.email_edit.text())
        logger.debug('About to emit userLoggedIn signal with: (%s).', email)
        self.userLoggedIn.emit(email)

    def on_forgotten_password(self, link=None):
        """Show the user the forgotten password page."""
        # Now use UbuntuKylin forum to setup account - 20141015 Robert
        # self.hide_overlay()
        # logger.debug('About to emit passwordForgotten signal')
        # self.passwordForgotten.emit()
        import webbrowser
        webbrowser.open(link)

    def on_create_account(self, link=None):
        """Show the user the account creation page."""
        # Now use UbuntuKylin forum to setup account - 20141015 Robert
        # self.hide_overlay()
        # logger.debug('About to emit createAccount signal')
        # self.createAccount.emit()
        import webbrowser
        webbrowser.open(link)


# -*- coding: utf-8 -*-
#
# Copyright 2010-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Utility modules that may find use outside ubuntu_kylin_sso."""

from __future__ import unicode_literals

import os
import sys

from dirspec.basedir import load_config_paths
from dirspec.utils import get_program_path

from twisted.python import procutils

from ubuntu_kylin_sso import (BACKEND_EXECUTABLE,
                        UI_EXECUTABLE_QT,
                        UI_PROXY_CREDS_DIALOG,
                        UI_SSL_DIALOG)
from ubuntu_kylin_sso.logger import setup_logging


logger = setup_logging("ubuntu_kylin_sso.utils")

BIN_SUFFIX = 'bin'
DATA_SUFFIX = 'data'

QSS_MAP = dict(win32=':/windows.qss',
               darwin=':/darwin.qss',
               linux=':/linux.qss')

# Setting linux as default if we don't find the
# platform as a key in the dictionary
PLATFORM_QSS = QSS_MAP.get(sys.platform, ":/linux.qss")

DARWIN_APP_NAMES = {BACKEND_EXECUTABLE: 'Ubuntu SSO Helper.app',
                    UI_EXECUTABLE_QT: 'Ubuntu Single Sign-On.app',
                    UI_SSL_DIALOG: 'Ubuntu SSO SSL Certificate.app',
                    UI_PROXY_CREDS_DIALOG: 'Ubuntu SSO Proxy Credentials.app'
                    }


def _get_dir(dir_name, dir_constant):
    """Return the absolute path to this project's 'dir_name' dir.

    Support symlinks, and priorize local (relative) 'dir_name' dir. If not
    found, return the value of the 'dir_constant'.

    """
    module = os.path.dirname(__file__)
    result = os.path.abspath(os.path.join(module, os.path.pardir,
                                          os.path.pardir, dir_name))
    logger.debug('_get_dir: trying use dir at %r (exists? %s)',
                  result, os.path.exists(result))
    if os.path.exists(result):
        logger.info('_get_dir: returning dir located at %r.', result)
        return result

    # otherwise, try to load 'dir_constant' from installation path
    try:
        __import__('ubuntu_kylin_sso.constants', None, None, [''])
        module = sys.modules.get('ubuntu_kylin_sso.constants')
        return getattr(module, dir_constant)
    except (ImportError, AttributeError):
        msg = '_get_dir: can not build a valid path. Giving up. ' \
              '__file__ is %r, constants module not available.'
        logger.error(msg, __file__)


def get_project_dir():
    """Return the absolute path to this project's data/ dir.

    Support symlinks, and priorize local (relative) data/ dir. If not
    found, return the value of the PROJECT_DIR.

    """
    result = _get_dir(dir_name=DATA_SUFFIX, dir_constant='PROJECT_DIR')
    assert result is not None, '%r dir can not be None.' % DATA_SUFFIX
    return result


def get_data_file(*args):
    """Return the absolute path to 'args' within project data dir."""
    return os.path.join(get_project_dir(), *args)


def get_bin_dir():
    """Return the absolute path to this project's bin/ dir.

    Support symlinks, and priorize local (relative) bin/ dir. If not
    found, return the value of the BIN_DIR.

    """
    result = _get_dir(dir_name=BIN_SUFFIX, dir_constant='BIN_DIR')
    assert result is not None, '%r dir can not be None.' % BIN_SUFFIX
    logger.info('get_bin_dir: returning dir located at %r.', result)
    return result


def get_bin_cmd(program_name):
    """Return a list of arguments to launch the given executable."""
    path = get_program_path(program_name,
                            fallback_dirs=[get_bin_dir()],
                            app_names=DARWIN_APP_NAMES)
    cmd_args = [path]

    # adjust cmd for platforms using buildout-generated python
    # wrappers
    if getattr(sys, 'frozen', None) is None:
        if sys.platform in ('darwin'):
            cmd_args.insert(0, 'python')
        elif sys.platform in ('win32'):
            cmd_args.insert(0, procutils.which("python.exe")[0])

    logger.debug('get_bin_cmd: returning %r', cmd_args)
    return cmd_args


def get_cert_dir():
    """Return directory containing certificate files."""

    if getattr(sys, "frozen", None) is not None:
        if sys.platform == "win32":
            ssl_cert_location = list(load_config_paths(
                    "ubuntuone"))[1]
        elif sys.platform == "darwin":
                main_app_dir = "".join(__file__.partition(".app")[:-1])
                main_app_resources_dir = os.path.join(main_app_dir,
                                                      "Contents",
                                                      "Resources")
                ssl_cert_location = main_app_resources_dir
    elif any(plat in sys.platform for plat in ("win32", "darwin")):
        pkg_dir = os.path.dirname(__file__)
        src_tree_path = os.path.dirname(os.path.dirname(pkg_dir))
        ssl_cert_location = os.path.join(src_tree_path,
                                         "data")
    else:
        ssl_cert_location = '/etc/ssl/certs'

    return ssl_cert_location


def ping_url(url, email, credentials):
    """Unused -- it was formerly used to let U1 know about new accounts."""



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Retrieve the proxy configuration from Gnome."""

import subprocess

from ubuntu_kylin_sso.logger import setup_logging

logger = setup_logging("ubuntu_kylin_sso.utils.webclient.gsettings")
GSETTINGS_CMDLINE = "gsettings list-recursively org.gnome.system.proxy"
CANNOT_PARSE_WARNING = "Cannot parse gsettings value: %r"


def parse_proxy_host(hostname):
    """Parse the host to get username and password."""
    username = None
    password = None
    if "@" in hostname:
        username, hostname = hostname.rsplit("@", 1)
        if ":" in username:
            username, password = username.split(":", 1)
    return hostname, username, password


def parse_manual_proxy_settings(scheme, gsettings):
    """Parse the settings for a given scheme."""
    host, username, pwd = parse_proxy_host(gsettings[scheme + ".host"])
    # if the user did not set a proxy for a type (http/https/ftp) we should
    # return None to ensure that it is not used
    if host == '':
        return None

    settings = {
        "host": host,
        "port": gsettings[scheme + ".port"],
    }
    if scheme == "http" and gsettings["http.use-authentication"]:
        username = gsettings["http.authentication-user"]
        pwd = gsettings["http.authentication-password"]
    if username is not None and pwd is not None:
        settings.update({
            "username": username,
            "password": pwd,
        })
    return settings


def get_proxy_settings():
    """Parse the proxy settings as returned by the gsettings executable."""
    output = subprocess.check_output(GSETTINGS_CMDLINE.split())
    gsettings = {}
    base_len = len("org.gnome.system.proxy.")
    # pylint: disable=E1103
    for line in output.split("\n"):
        try:
            path, key, value = line.split(" ", 2)
        except ValueError:
            continue
        if value.startswith("'"):
            parsed_value = value[1:-1]
        elif value.startswith(('[', '@')):
            parsed_value = value
        elif value in ('true', 'false'):
            parsed_value = (value == 'true')
        elif value.isdigit():
            parsed_value = int(value)
        else:
            logger.warning(CANNOT_PARSE_WARNING, value)
            parsed_value = value
        relative_key = (path + "." + key)[base_len:]
        gsettings[relative_key] = parsed_value
    mode = gsettings["mode"]
    if mode == "none":
        settings = {}
    elif mode == "manual":
        settings = {}
        for scheme in ["http", "https"]:
            scheme_settings = parse_manual_proxy_settings(scheme, gsettings)
            if scheme_settings is not None:
                settings[scheme] = scheme_settings
    else:
        # If mode is automatic the PAC javascript should be interpreted
        # on each request. That is out of scope so it's ignored for now
        settings = {}

    return settings



# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Helpful code for testing on linux."""

from twisted.internet import defer

from ubuntuone.devtools.testcases.dbus import DBusTestCase

# pylint complains about callables
# pylint: disable=E1102


class BaseServiceTestCase(DBusTestCase):
    """Set the ipc to a random port for this instance."""

    client_class = None  # the BaseClient instance
    service_class = None  # the BaseService instance

    @defer.inlineCallbacks
    def setUp(self):
        yield super(BaseServiceTestCase, self).setUp()
        self.service = None
        self.client = None
        if self.service_class is not None:
            self.service = self.service_class()
            self.client = self.client_class()

            # pylint: disable=E1102
            yield self.service.start()
            self.addCleanup(self.service.shutdown)
            yield self.client.connect()
            self.addCleanup(self.client.disconnect)



# -*- coding: utf-8 -*-
#
# Author: Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Common functionality used by the UI modules."""

from __future__ import unicode_literals

import re

from ubuntu_kylin_sso.utils import compat
from ubuntu_kylin_sso.utils.ui import (
    PASSWORD_DIGIT,
    PASSWORD_LENGTH,
    PASSWORD_MATCH,
    PASSWORD_MUST_CONTAIN,
    PASSWORD_UPPER,
)

# all the text + styles that are used in the gui
BAD = '<img src=":/password_hint_warning.png" /><small> %s </small>'
GOOD = '<img src=":/password_hint_ok.png" /><small> %s </small>'
NORMAL = '<small> %s </small>'


def password_assistance(line_edit, assistance, icon_type=BAD):
    """Show help for the password field."""
    text1 = compat.text_type(line_edit.text())
    label_text = ["<b>%s</b>" % PASSWORD_MUST_CONTAIN, ]

    # weaken password requirement. Robert
    if len(text1) < 8:
        sign = icon_type
    else:
        sign = GOOD
    label_text.append(sign % PASSWORD_LENGTH)

    # if re.search('[A-Za-z]', text1) is None:
    #     sign = icon_type
    # else:
    #     sign = GOOD
    # label_text.append(sign % PASSWORD_UPPER)

    # if re.search('[\d+]', text1) is None:
    #     sign = icon_type
    # else:
    #     sign = GOOD
    # label_text.append(sign % PASSWORD_DIGIT)

    assistance.setText("<br>".join(label_text))


def password_check_match(line_edit, line_edit_confirm, assistance):
    """Check if passwords match, otherwise show a message."""
    password_assistance(line_edit, assistance)
    label_text = compat.text_type(assistance.text())
    text1 = compat.text_type(line_edit.text())
    text2 = compat.text_type(line_edit_confirm.text())
    if text1 != text2:
        label_text += "<br>" + BAD % PASSWORD_MATCH
    assistance.setText(label_text)


def password_default_assistance(assistance):
    """Show default help for the password field."""
    label_text = ["<b>%s</b>" % PASSWORD_MUST_CONTAIN, ]
    label_text.append(NORMAL % PASSWORD_LENGTH)
    # weaken the password requirement. Robert
    # label_text.append(NORMAL % PASSWORD_UPPER)
    # label_text.append(NORMAL % PASSWORD_DIGIT)
    assistance.setText("<br>".join(label_text))


def check_as_invalid(line_edit):
    """Set QLineEdit's formError property as True, refresh the style."""
    line_edit.setProperty("formError", True)
    line_edit.style().unpolish(line_edit)
    line_edit.style().polish(line_edit)


def check_as_valid(line_edit):
    """Set QLineEdit's formError property as False, refresh the style."""
    line_edit.setProperty("formError", False)
    line_edit.style().unpolish(line_edit)
    line_edit.style().polish(line_edit)



# -*- coding: utf-8 -*-
#
# Copyright 2010-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Implementation of network state detection."""

import dbus

from twisted.internet import defer

from ubuntu_kylin_sso.networkstate import NetworkFailException
from ubuntu_kylin_sso.networkstate.networkstates import (
    ONLINE, OFFLINE,
    NM_STATE_CONNECTING_LIST,
    NM_STATE_CONNECTED_LIST,
    NM_STATE_DISCONNECTED_LIST,
)
from ubuntu_kylin_sso.logger import setup_logging
logger = setup_logging("ubuntu_kylin_sso.networkstate")

NM_DBUS_INTERFACE = "org.freedesktop.NetworkManager"
NM_DBUS_OBJECTPATH = "/org/freedesktop/NetworkManager"


class NetworkManagerState(object):
    """Checks the state of NetworkManager thru DBus."""

    def __init__(self, result_cb, dbus_module=dbus):
        """Initialize this instance with a result and error callbacks."""
        self.result_cb = result_cb
        self.dbus = dbus_module
        self.state_signal = None

    def call_result_cb(self, state):
        """Return the state thru the result callback."""
        self.result_cb(state)

    def got_state(self, state):
        """Called by DBus when the state is retrieved from NM."""
        if state in NM_STATE_CONNECTED_LIST:
            self.call_result_cb(ONLINE)
        elif state in NM_STATE_CONNECTING_LIST:
            logger.debug("Currently connecting, waiting for signal")
        else:
            self.call_result_cb(OFFLINE)

    def got_error(self, error):
        """Called by DBus when the state is retrieved from NM."""
        # Assuming since Network Manager is not running,
        # the user has connected in some other way
        logger.error("Error contacting NetworkManager: %s" %
                         str(error))
        self.call_result_cb(ONLINE)

    def state_changed(self, state):
        """Called when a signal is emmited by Network Manager."""
        if int(state) in NM_STATE_CONNECTED_LIST:
            self.call_result_cb(ONLINE)
        elif int(state) in NM_STATE_DISCONNECTED_LIST:
            self.call_result_cb(OFFLINE)
        else:
            logger.debug("Not yet connected: continuing to wait")

    def find_online_state(self):
        """Get the network state and return it thru the set callback."""
        try:
            sysbus = self.dbus.SystemBus()
            nm_proxy = sysbus.get_object(NM_DBUS_INTERFACE,
                                         NM_DBUS_OBJECTPATH,
                                         follow_name_owner_changes=True)
            nm_if = self.dbus.Interface(nm_proxy, NM_DBUS_INTERFACE)
            self.state_signal = nm_if.connect_to_signal(
                        signal_name="StateChanged",
                        handler_function=self.state_changed,
                        dbus_interface=NM_DBUS_INTERFACE)
            nm_proxy.Get(NM_DBUS_INTERFACE, "State",
                         reply_handler=self.got_state,
                         error_handler=self.got_error)
        except Exception as e:
            self.got_error(e)


def is_machine_connected():
    """Return a deferred that when fired, returns if the machine is online."""
    d = defer.Deferred()

    def got_state(state):
        """The state was retrieved from the Network Manager."""
        if type(state) is not type(ONLINE):
            logger.exception("bad callback argument in is_machine_connected")
            raise NetworkFailException()
        result = (state == ONLINE)
        d.callback(result)

    try:
        network = NetworkManagerState(got_state)
        network.find_online_state()
    except Exception as e:
        logger.exception('is_machine_connected failed with:')
        d.errback(NetworkFailException(e))

    return d



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utility classes used for testing main on linux."""

from twisted.internet import defer

from ubuntu_kylin_sso import main
from ubuntuone.devtools.testcases.dbus import DBusTestCase


class BaseTestCase(DBusTestCase):
    """The base test case with platform specific support."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(BaseTestCase, self).setUp()

        self.sso_service = main.UbuntuSSOService()
        yield self.sso_service.start()
        self.addCleanup(self.sso_service.shutdown)

        self.sso_client = yield main.get_sso_client()
        self.addCleanup(self.sso_client.disconnect)



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""A webclient backend that uses libsoup."""

import httplib

from twisted.internet import defer

from ubuntu_kylin_sso.logger import setup_logging
from ubuntu_kylin_sso.utils.webclient.common import (
    BaseWebClient,
    HeaderDict,
    Response,
    ProxyUnauthorizedError,
    UnauthorizedError,
    WebClientError,
)

URI_ANONYMOUS_TEMPLATE = "http://{host}:{port}/"
URI_USERNAME_TEMPLATE = "http://{username}:{password}@{host}:{port}/"

logger = setup_logging("ubuntu_kylin_sso.utils.webclient.libsoup")


class WebClient(BaseWebClient):
    """A webclient with a libsoup backend."""

    def __init__(self, *args, **kwargs):
        """Initialize this instance."""
        super(WebClient, self).__init__(*args, **kwargs)
        # pylint: disable=E0611,F0401
        from gi.repository import Soup, SoupGNOME
        self.soup = Soup
        self.session = Soup.SessionAsync()
        self.session.add_feature(SoupGNOME.ProxyResolverGNOME())
        self.session.connect("authenticate", self._on_authenticate)

    def _on_message(self, session, message, d):
        """Handle the result of an http message."""
        logger.debug('_on_message status code is %s', message.status_code)
        if message.status_code == httplib.OK:
            headers = HeaderDict()
            response_headers = message.get_property("response-headers")
            add_header = lambda key, value, _: headers[key].append(value)
            response_headers.foreach(add_header, None)
            content = message.response_body.flatten().get_data()
            response = Response(content, headers)
            d.callback(response)
        elif message.status_code == httplib.UNAUTHORIZED:
            e = UnauthorizedError(message.reason_phrase)
            d.errback(e)
        elif message.status_code == httplib.PROXY_AUTHENTICATION_REQUIRED:
            e = ProxyUnauthorizedError(message.reason_phrase)
            d.errback(e)
        else:
            e = WebClientError(message.reason_phrase)
            d.errback(e)

    @defer.inlineCallbacks
    def _on_authenticate(self, session, message, auth, retrying, data=None):
        """Handle the "authenticate" signal."""
        self.session.pause_message(message)
        try:
            logger.debug('_on_authenticate: message status code is %s',
                         message.status_code)
            if not retrying and self.username and self.password:
                auth.authenticate(self.username, self.password)
            if auth.is_for_proxy():
                logger.debug('_on_authenticate auth is for proxy.')
                got_creds = yield self.request_proxy_auth_credentials(
                    self.session.props.proxy_uri.host,
                    retrying)
                if got_creds:
                    logger.debug('Got proxy credentials from user.')
                    auth.authenticate(self.proxy_username, self.proxy_password)
        finally:
            self.session.unpause_message(message)

    @defer.inlineCallbacks
    def _on_proxy_authenticate(self, failure, iri, method="GET",
                extra_headers=None, oauth_credentials=None, post_content=None):
        """Deal with wrong settings."""
        failure.trap(ProxyUnauthorizedError)
        logger.debug('Proxy settings are wrong.')
        got_creds = yield self.request_proxy_auth_credentials(
            self.session.props.proxy_uri.host,
            True)
        if got_creds:
            settings = dict(host=self.session.props.proxy_uri.host,
                            port=self.session.props.proxy_uri.port,
                            username=self.proxy_username,
                            password=self.proxy_password)
            self.force_use_proxy(settings)
            response = yield self.request(iri, method, extra_headers,
                                         oauth_credentials, post_content)
            defer.returnValue(response)

    @defer.inlineCallbacks
    def request(self, iri, method="GET", extra_headers=None,
                oauth_credentials=None, post_content=None):
        """Return a deferred that will be fired with a Response object."""
        uri = self.iri_to_uri(iri)
        headers = yield self.build_request_headers(uri, method, extra_headers,
                                                   oauth_credentials)
        d = defer.Deferred()
        message = self.soup.Message.new(method, uri)

        for key, value in headers.items():
            message.request_headers.append(key, value)

        if post_content:
            message.request_body.append(post_content)

        self.session.queue_message(message, self._on_message, d)
        d.addErrback(self._on_proxy_authenticate, iri, method, extra_headers,
                     oauth_credentials, post_content)
        response = yield d
        defer.returnValue(response)

    def force_use_proxy(self, settings):
        """Setup this webclient to use the given proxy settings."""
        # pylint: disable=W0511
        proxy_uri = self.get_proxy_uri(settings)
        self.session.set_property("proxy-uri", proxy_uri)

    def get_proxy_uri(self, settings):
        """Get a Soup.URI for the proxy, or None if disabled."""
        if "host" in settings and "port" in settings:
            template = URI_ANONYMOUS_TEMPLATE
            if "username" in settings and "password" in settings:
                template = URI_USERNAME_TEMPLATE
            uri = template.format(**settings)
            return self.soup.URI.new(uri)
        else:
            # If the proxy host is not set, use no proxy
            return None

    def shutdown(self):
        """End the soup session for this webclient."""
        self.session.abort()



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Test the platform-specific translation functions."""

import os
import sys

from twisted.internet import defer
from ubuntuone.devtools.testcases import TestCase, skipIfNotOS
from ubuntu_kylin_sso.utils import translation

TEST_DOMAIN = 'test-domain'
TEST_LANG_DEFAULTS_EN_FIRST = ['en', 'pt-PT']
TEST_LANG_DEFAULTS = ['es', 'en', 'pt-PT']
TEST_FALLBACK_PATH = 'test-path/to/mofiles'
TEST_FROZEN_PATH = 'frozen-test-path/to/mofiles'


class MockGettextTranslations(object):
    """Mock translations to test properties"""
    ugettext = 'ugettext'
    gettext = 'gettext'


class MockNSUserDefaults(object):
    """Mock defaults for _get_languages on darwin."""

    def standardUserDefaults(self):
        return {'AppleLanguages': TEST_LANG_DEFAULTS}


class TranslationsTestCase(TestCase):
    """Test getting the right gettext translations."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(TranslationsTestCase, self).setUp()
        self._called = []

    def _set_called(self, *args, **kwargs):
        """call recorder"""
        self._called.append((args, kwargs))
        return MockGettextTranslations()

    def test_import(self):
        """Test whether import translation defines _ as a builtin."""
        import ubuntu_kylin_sso.utils.translation
        assert ubuntu_kylin_sso.utils.translation
        self.assertFalse('_' in __builtins__)

    @skipIfNotOS('darwin', 'Test requires pyobjc-Cocoa')
    def test_get_languages_darwin(self):
        """Test getting the user's list of preferred languages."""
        import Cocoa
        assert Cocoa
        self.patch(Cocoa, 'NSUserDefaults', MockNSUserDefaults())
        langs = translation._get_languages()
        self.assertEqual(langs, TEST_LANG_DEFAULTS)

    def test_get_languages_linux(self):
        """Test that we will use gettext defaults on linux."""
        self.patch(sys, 'platform', 'linux2')
        langs = translation._get_languages()
        self.assertEqual(langs, None)

    def test_get_translations_data_path_darwin_frozen(self):
        """Test getting the location of the compiled translation files."""
        self.patch(sys, 'platform', 'darwin')
        sys.frozen = 'yes'
        self.addCleanup(delattr, sys, 'frozen')
        self.patch(translation, '__file__',
                   os.path.join('path', 'to', 'Main.app', 'ignore', 'me'))

        path = translation._get_translations_data_path()

        self.assertEqual(path, os.path.join('path', 'to', 'Main.app',
                                            'Contents', 'Resources',
                                            'translations'))

    def test_get_translations_data_path_darwin_unfrozen_nofallback(self):
        """Test that we use gettext defaults on darwin when not frozen."""
        self.patch(sys, 'platform', 'darwin')
        path = translation._get_translations_data_path()
        self.assertEqual(path, None)

    def test_get_translations_data_path_darwin_unfrozen_fallback(self):
        """Test that we use fallback on darwin when not frozen."""
        self.patch(sys, 'platform', 'darwin')
        expected = "a-test-path"
        path = translation._get_translations_data_path(expected)
        self.assertEqual(path, expected)

    def test_get_translations_data_path_linux(self):
        """Test that we use gettext defaults on linux."""
        path = translation._get_translations_data_path()
        self.assertEqual(path, None)

    def _call_get_gettext(self, platform, py_version, fallback=None):
        """Helper function to patch and call translation.get_gettext."""
        self.patch(sys, 'platform', platform)
        self.patch(sys, 'version_info', py_version)
        self.patch(translation.gettext, 'translation', self._set_called)

        if fallback:
            g_func = translation.get_gettext(TEST_DOMAIN, fallback)
        else:
            g_func = translation.get_gettext(TEST_DOMAIN)

        if py_version == (2,):
            self.assertEqual(g_func, 'ugettext')
        else:
            self.assertEqual(g_func, 'gettext')

    def test_get_gettext_linux_py2(self):
        """test get_gettext on linux py2"""
        self._call_get_gettext('linux2', py_version=(2,))

    def test_get_gettext_linux_py3(self):
        """test get_gettext on linux py3"""
        self._call_get_gettext('linux2', py_version=(3,))

    def _call_get_gettext_nonlinux(self, frozen, py_version,
                                   lang_en_first=False):
        """Helper function for non-linux runs of get_gettext."""
        if lang_en_first:
            lang_rv = TEST_LANG_DEFAULTS_EN_FIRST
        else:
            lang_rv = TEST_LANG_DEFAULTS
        self.patch(translation, '_get_languages', lambda: lang_rv)
        if frozen:
            expected_path = TEST_FROZEN_PATH
            sys.frozen = 'yes'
            self.addCleanup(delattr, sys, 'frozen')
        else:
            expected_path = TEST_FALLBACK_PATH

        self.patch(translation, '_get_translations_data_path',
                   lambda x: expected_path)

        self._call_get_gettext('notlinux', py_version)

        # This tests a special case if 'en' is the first language, we
        # don't give the path, so we will get the fallback
        # NullTranslation instance that will use the keys and not look
        # for en translations, which we do not ship.
        if lang_en_first:
            expected_arg = ((TEST_DOMAIN,), {'fallback': True})
            self.assertEqual(self._called, [expected_arg])
        else:
            # Check for lang_rv[:1] (must be a list) because we only
            # send first language, in order to fall back to 'en' even
            # though we don't ship an en.mo
            expected_arg = ((TEST_DOMAIN, expected_path),
                              {'languages': lang_rv[:1],
                               'fallback': True})

            self.assertEqual(self._called, [expected_arg])

    def test_get_gettext_nonlinux_frozen_py2(self):
        """test get_gettext on nonlinux frozen and py2"""
        self._call_get_gettext_nonlinux(frozen=True, py_version=(2,))

    def test_get_gettext_nonlinux_frozen_py3(self):
        """test get_gettext on nonlinux frozen and py3"""
        self._call_get_gettext_nonlinux(frozen=True, py_version=(3,))

    def test_get_gettext_nonlinux_unfrozen_py2(self):
        """test get_gettext on nonlinux un-frozen and py2"""
        self._call_get_gettext_nonlinux(frozen=False, py_version=(2,))

    def test_get_gettext_nonlinux_unfrozen_py3(self):
        """test get_gettext on nonlinux un-frozen and py3"""
        self._call_get_gettext_nonlinux(frozen=False, py_version=(3,))

    def test_get_gettext_nonlinux_frozen_py2_enfirst(self):
        """test get_gettext returning nulltranslations when lang[0] = en"""
        self._call_get_gettext_nonlinux(frozen=True,
                                        py_version=(2,),
                                        lang_en_first=True)

    def test_get_gettext_darwin_unfrozen_fallback(self):
        """test using fallback path from source"""
        self.patch(translation, '_get_languages', lambda: ['not-en'])
        self._call_get_gettext('darwin', (2,), TEST_FALLBACK_PATH)
        self.assertEqual(self._called, [((TEST_DOMAIN, TEST_FALLBACK_PATH),
                                       {'languages': ['not-en'],
                                        'fallback': True})])

    def test_get_gettext_win32_unfrozen_fallback(self):
        """test using fallback path from source"""
        self.patch(translation, '_get_languages', lambda: ['not-en'])
        self._call_get_gettext('win32', (2,), TEST_FALLBACK_PATH)
        self.assertEqual(self._called, [((TEST_DOMAIN, TEST_FALLBACK_PATH),
                                       {'languages': ['not-en'],
                                        'fallback': True})])



#
# Copyright 2009-2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Ubuntu Single Sign On client code."""

import os


# Host URL changing environment variables
# SSO_AUTH_BASE_URL = os.environ.get(
#     'SSO_AUTH_BASE_URL', 'https://login.ubuntu.com')
# SSO_UONE_BASE_URL = os.environ.get(
#     'SSO_UONE_BASE_URL', 'https://one.ubuntu.com')
SSO_AUTH_BASE_URL = 'https://login.ubuntukylin.com'#'http://login.ubuntukylin.com:8001'#'http://0.0.0.0:8000' #'http://192.168.30.12'
SSO_UONE_BASE_URL = '' #'http://0.0.0.0:8000'

# DBus constants
DBUS_BUS_NAME = "com.ubuntukylin.sso"

DBUS_ACCOUNT_PATH = "/com/ubuntukylin/sso/accounts"
DBUS_IFACE_USER_NAME = "com.ubuntukylin.sso.UserManagement"

DBUS_CREDENTIALS_PATH = "/com/ubuntukylin/sso/credentials"
DBUS_CREDENTIALS_IFACE = "com.ubuntukylin.sso.CredentialsManagement"

NO_OP = lambda *args, **kwargs: None

# return codes for UIs
USER_SUCCESS = 0
USER_CANCELLATION = 10
EXCEPTION_RAISED = 11

BACKEND_EXECUTABLE = 'ubuntu-kylin-sso-login'

# available UIs
UI_SSL_DIALOG = 'ubuntu-kylin-sso-ssl-certificate-qt'
UI_EXECUTABLE_QT = 'ubuntu-kylin-sso-login-qt'
UI_PROXY_CREDS_DIALOG = 'ubuntu-kylin-sso-proxy-creds-qt'



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Reset Password page UI."""

from functools import partial

from PyQt4.QtCore import Qt, SIGNAL, pyqtSignal
from PyQt4.QtGui import QApplication

from ubuntu_kylin_sso import NO_OP
from ubuntu_kylin_sso.logger import setup_gui_logging
from ubuntu_kylin_sso.qt import build_general_error_message, common
from ubuntu_kylin_sso.qt.sso_wizard_page import SSOWizardEnhancedEditPage
from ubuntu_kylin_sso.qt.ui.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_kylin_sso.utils import compat
from ubuntu_kylin_sso.utils.ui import (
    is_min_required_password,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    RESET_CODE_ENTRY,
    RESET_PASSWORD,
    RESET_TITLE,
    RESET_SUBTITLE,
)


logger = setup_gui_logging('ubuntu_kylin_sso.reset_password_page')


class ResetPasswordPage(SSOWizardEnhancedEditPage):
    """Widget used to allow the user change his password."""

    ui_class = Ui_ResetPasswordPage
    passwordChanged = pyqtSignal(compat.text_type)

    @property
    def _signals(self):
        """The signals to connect to the backend."""
        result = {
            'PasswordChanged':
                self._filter_by_app_name(self.on_password_changed),
            'PasswordChangeError':
                self._filter_by_app_name(self.on_password_change_error),
        }
        return result

    def focus_changed(self, old, now):
        """Check who has the focus to activate password popups if necessary."""
        if now == self.ui.password_line_edit:
            self.ui.password_assistance.setVisible(True)
            common.password_default_assistance(self.ui.password_assistance)
        elif now == self.ui.confirm_password_line_edit:
            common.password_check_match(self.ui.password_line_edit,
                                      self.ui.confirm_password_line_edit,
                                      self.ui.password_assistance)

    # Invalid name "initializePage"
    # pylint: disable=C0103

    def initializePage(self):
        """Extends QWizardPage initializePage method."""
        logger.debug('initializePage - About to show ResetPasswordPage')
        super(ResetPasswordPage, self).initializePage()
        self.ui.gridLayout.setAlignment(Qt.AlignLeft)
        common.password_default_assistance(self.ui.password_assistance)
        self.ui.password_assistance.setVisible(False)
        self.setTitle(RESET_TITLE)
        self.setSubTitle(RESET_SUBTITLE)
        self.ui.password_label.setText(PASSWORD1_ENTRY)
        self.ui.confirm_password_label.setText(PASSWORD2_ENTRY)
        self.ui.reset_code.setText(RESET_CODE_ENTRY)

        self.ui.reset_password_button.setDefault(True)
        self.ui.reset_password_button.setEnabled(False)

    def showEvent(self, event):
        """Connect focusChanged signal from the application."""
        super(ResetPasswordPage, self).showEvent(event)
        self.connect(QApplication.instance(),
            SIGNAL("focusChanged(QWidget*, QWidget*)"),
            self.focus_changed)

    def hideEvent(self, event):
        """Disconnect the focusChanged signal when the page change."""
        super(ResetPasswordPage, self).hideEvent(event)
        try:
            self.disconnect(QApplication.instance(),
                SIGNAL("focusChanged(QWidget*, QWidget*)"),
                self.focus_changed)
        except TypeError:
            pass

    # pylint: enable=C0103

    def _set_translated_strings(self):
        """Translate the diff strings used in the app."""
        self.ui.reset_password_button.setText(RESET_PASSWORD)
        self.setSubTitle(PASSWORD_HELP)

    def _connect_ui(self):
        """Connect the different ui signals."""
        self.ui.password_line_edit.textEdited.connect(
            lambda: common.password_assistance(self.ui.password_line_edit,
                                                 self.ui.password_assistance,
                                                 common.NORMAL))
        self.ui.confirm_password_line_edit.textEdited.connect(
            lambda: common.password_check_match(self.ui.password_line_edit,
                                      self.ui.confirm_password_line_edit,
                                      self.ui.password_assistance))

        self.ui.reset_password_button.clicked.connect(
                                                    self.set_new_password)
        self.ui.reset_code_line_edit.textChanged.connect(self._validate)
        self.ui.password_line_edit.textChanged.connect(self._validate)
        self.ui.confirm_password_line_edit.textChanged.connect(
            self._validate)

        self._add_line_edits_validations()

    def _validate(self):
        """Enable the submit button if data is valid."""
        enabled = True
        code = compat.text_type(self.ui.reset_code_line_edit.text())
        password = compat.text_type(self.ui.password_line_edit.text())
        confirm_password = compat.text_type(
            self.ui.confirm_password_line_edit.text())
        if not is_min_required_password(password):
            enabled = False
        elif not self.is_correct_password_confirmation(confirm_password):
            enabled = False
        elif not code:
            enabled = False
        self.ui.reset_password_button.setEnabled(enabled)

    def _add_line_edits_validations(self):
        """Add the validations to be use by the line edits."""
        self.set_line_edit_validation_rule(
            self.ui.password_line_edit,
            is_min_required_password)
        self.set_line_edit_validation_rule(
            self.ui.confirm_password_line_edit,
            self.is_correct_password_confirmation)
        # same as the above case, lets connect a signal to a signal
        self.ui.password_line_edit.textChanged.connect(
            self.ui.confirm_password_line_edit.textChanged.emit)

    def on_password_changed(self, app_name, email):
        """Let user know that the password was changed."""
        logger.info('ResetPasswordPage.on_password_changed for %s, email: %s',
            app_name, email)
        self.hide_overlay()
        email = compat.text_type(
            self.wizard().forgotten.ui.email_line_edit.text())
        self.passwordChanged.emit(email)

    def on_password_change_error(self, app_name, error):
        """Let the user know that there was an error."""
        logger.error('Got error changing password for %s, error: %s',
                     self.app_name, error)
        self.show_error(build_general_error_message(error))

    def set_new_password(self):
        """Request a new password to be set."""
        self.hide_error()
        email = compat.text_type(
                    self.wizard().forgotten.ui.email_line_edit.text())
        code = compat.text_type(self.ui.reset_code_line_edit.text())
        password = compat.text_type(self.ui.password_line_edit.text())
        logger.info('Setting new password for %r and email %r with code %r',
                    self.app_name, email, code)
        args = (self.app_name, email, code, password)
        f = self.backend.set_new_password
        error_handler = partial(self._handle_error, f,
            self.on_password_change_error)
        self.show_overlay()
        f(*args, reply_handler=NO_OP, error_handler=error_handler)

    def is_correct_password_confirmation(self, password):
        """Return if the password is correct."""
        return compat.text_type(self.ui.password_line_edit.text()) == password



# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Tests for the EnhancedCheckBox widget."""

from PyQt4 import QtGui, QtCore

from ubuntu_kylin_sso.qt import enhanced_check_box
from ubuntu_kylin_sso.qt.tests import BaseTestCase


class EnhancedCheckBoxTestCase(BaseTestCase):
    """Test the EnhancedCheckBox code."""

    def test_init(self):
        """Check the initial state of EnhancedCheckBox."""
        check = enhanced_check_box.EnhancedCheckBox("text")
        self.assertEqual(check.text(), "text")
        self.assertTrue(check.text_label.openExternalLinks())
        self.assertEqual(check.receivers(
            QtCore.SIGNAL('stateChanged(int)')), 1)

    def test_set_text(self):
        """Check if override setText and text methods works properly."""
        check = enhanced_check_box.EnhancedCheckBox()
        self.assertEqual(check.text_label.text(), "")
        check.setText("text")
        self.assertEqual(check.text(), "text")
        self.assertEqual(check.text(), check.text_label.text())

    def test_enhanced_check_size_adjust_with_small_height(self):
        """Check if the size of the EnhancedCheckBox is adjusted correctly."""
        text = 't' * 200
        height = 10
        widget = QtGui.QWidget()
        widget.setFixedSize(200, height)
        check = enhanced_check_box.EnhancedCheckBox(text, widget)
        self.assertTrue(check.height() > height)

    def test_enhanced_check_size_adjust_with_big_height(self):
        """Check if the size of the EnhancedCheckBox is adjusted correctly."""
        text = 't' * 20
        height = 200
        widget = QtGui.QWidget()
        widget.setFixedSize(200, height)
        check = enhanced_check_box.EnhancedCheckBox(text, widget)
        self.assertTrue(check.height() < height)



# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# In addition, as a special exception, the copyright holders give
# permission to link the code of portions of this program with the
# OpenSSL library under certain conditions as described in each
# individual source file, and distribute linked combinations
# including the two.
# You must obey the GNU General Public License in all respects
# for all of the code used other than OpenSSL.  If you modify
# file(s) with this exception, you may extend this exception to your
# version of the file(s), but you are not obligated to do so.  If you
# do not wish to do so, delete this exception statement from your
# version.  If you delete this exception statement from all source
# files in the program, then also delete it here.
"""Widget written in Qt that works as a GtkArrow."""

from PyQt4.QtGui import QPainter, QStyle, QStyleOption, QWidget


class QArrow(QWidget):
    """Custom widget."""

    UP = 0
    DOWN = 1
    LEFT = 2
    RIGHT = 3

    def __init__(self, direction, parent=None):
        """Create a new instance."""
        super(QArrow, self).__init__(parent)
        self._set_direction(direction)
        self.setFixedWidth(16)

    # pylint: disable=C0103
    def paintEvent(self, event):
        """Paint the widget."""
        opt = QStyleOption()
        opt.initFrom(self)
        painter = QPainter(self)
        if self._direction == QArrow.UP:
            primitive = QStyle.PE_IndicatorArrowUp
        elif self._direction == QArrow.DOWN:
            primitive = QStyle.PE_IndicatorArrowDown
        elif self._direction == QArrow.LEFT:
            primitive = QStyle.PE_IndicatorArrowLeft
        else:
            primitive = QStyle.PE_IndicatorArrowRight
        painter.translate(-5, 0)
        painter.setViewTransformEnabled(True)
        self.style().drawPrimitive(primitive, opt, painter, self)
    # pylint: enable=C0103

    def _get_direction(self):
        """Return the direction used."""
        return self._direction

    # pylint: disable=W0201
    def _set_direction(self, direction):
        """Set the direction."""
        if direction not in (QArrow.UP, QArrow.DOWN,
                             QArrow.LEFT, QArrow.RIGHT):
            raise ValueError('Wrong arrow direction.')
        self._direction = direction
        self.repaint()
    # pylint: enable=W0201

    direction = property(_get_direction, _set_direction)
